package edu.cs2335.lab7.client;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferStrategy;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import edu.cs2335.lab7.creature.DWSpecies;
import edu.cs2335.lab7.data.CreatureData;
import edu.cs2335.lab7.data.MapImage;
import edu.cs2335.lab7.server.GameData;

/**
 * @author thaile
 * This Battle runs the clients side simulator that 
 * renders the battle to the screen base on data from
 * the server or local battle simulator.
 */
public class Battle extends Canvas implements Runnable {
    /** Object that running battle */
    private Object display;
    /** Queue of cmds for the creature to preformed */
    private ArrayList cmdQueue;
    /** DATA: species 1 */
    private DWSpecies species1;
    /** DATA: species 2 */
    private DWSpecies species2;
    /** DATA: The map renderer class */
    private MapRender map;
    /** DATA: Array of creature renderers */
    private CreatureRender[] creatures;
    /** DATA: for createing a board to draw on */
    private BufferStrategy strategy;
    /** DATA: last time checked */
    private long oldTime = 0;
    /** DATA: the winner */
    private String winner;
    
    //Canvas doesn't want to work with JScrollPane so we have to 
    //do manually scrolling by hand instead    
    /** DATA: for the drawing offset x-axis */
    private int xOff = 0;
    /** DATA: for the drawing offset y-axis */
    private int yOff = 0;
    
    /** FLAG: for the game running */
    private boolean playing = false;
    /** FLAG: ready for next creature move */
    private boolean readyNext = true;
    /** FLAG: waiting for server data to come */
    private boolean serverWait = true;
    /** FLAG: rotate bit */
    private boolean rotate = true;
    
    /**
     * Creates a new canvas for the battle
     * @param display the GUI thats using battle
     */
    public Battle(Object display) {
        this.display = display;
        cmdQueue = new ArrayList();
        //Setting up the graphical part
        setIgnoreRepaint(true);
        setBounds(0, 0, 480, 480);
    }
    
    
    /**
     * Initallizes the display for the battle
     * @param data the data about the battle
     */
    public void initBattle(GameData data) {
        // Setting up the battle info 
        species1 = data.getSpecies1();
        species2 = data.getSpecies2();
        creatures = new CreatureRender[data.getNumcreatures() * 2];
        
        if (data == null) {
            System.out.println("dat anull ");
        }
        MapImage mapi = data.getMap();
        if (mapi == null) {
            System.out.println("map null");
        }
        ImageIcon icon = mapi.getDirTile();
        System.out.println(icon.getIconHeight());
        if (icon == null) {
            System.out.println("dirtile null");
        }
        
        Image dirTile = data.getMap().getDirTile().getImage();
        map = new MapRender(data.getMap());
        
        CreatureData[] cInfo = data.getData();
        
        for (int i = 0; i < data.getNumcreatures() * 2; i++) {
            if (cInfo[i].getSpecies().equals(species1.getSpeciesType())) {
                creatures[i] = new CreatureRender(this, species1, cInfo[i]);
            } else {
                creatures[i] = new CreatureRender(this, species2, cInfo[i]);
            }
                creatures[i].setDirTile(dirTile);
        }

        //this enables scroll if map size is > 15
        if (display instanceof LocalBattle) {
            ((LocalBattle) display).scrollOn(data.getMap().getMap().length);
        } else if (display instanceof ServerBattle) {
            ((ServerBattle) display).scrollOn(data.getMap().getMap().length);
        }
            
        createBufferStrategy(2);
        strategy = getBufferStrategy();
        playing = true;
        
    }
    
    /**
     * Are we playing yet
     * @return true or false
     */    
    public boolean isPlaying() {
        return playing;
    }
    
    /**
     * This is what renders the game to the screen
     * 
     */
    public void run() {
        long time;
        while (playing) {
            //get the time since last loop through
            time = System.currentTimeMillis() - oldTime;
            oldTime = System.currentTimeMillis();
            
            Graphics2D board = (Graphics2D) strategy.getDrawGraphics();
            board.setColor(Color.black);
            board.fillRect(0, 0, 480, 480);
            
            //draw the board first 
            map.drawMap(board, xOff, yOff);
            
            //draw the creature next
            for (int i = 0; i < creatures.length; i++) {
                creatures[i].move(time);
                creatures[i].render(board, xOff, yOff);
            }
            
            //handle network cmds
            if (readyNext && !serverWait) {
                try {
                    
                    synchronized (cmdQueue) {
                        processCmd((String) cmdQueue.remove(0));  
                    }
                    readyNext = false;
                    if (cmdQueue.isEmpty()) {
                        serverWait = true;
                        if (display instanceof ServerBattle) {
                            ((ServerBattle) display).sendReady();
                        }
                    }
                } catch (IndexOutOfBoundsException e) {
                    //send msg ready for next turn to server
                    if (display instanceof ServerBattle) {
                        ((ServerBattle) display).sendReady();
                    }
                    serverWait = true; // reset
                }
            }
            
            
            // done drawing the board now show it

            board.dispose();
            strategy.show();
            
            try { 
                Thread.sleep(100); 
            } catch (Exception e) {
                break;
            }
        }
        
        if (display instanceof LocalBattle) {
            ((LocalBattle) display).gameOver();
        }

        if (!winner.equals("|none|")) {
            JOptionPane.
            showConfirmDialog(this,
                          "The winner " + winner,
                          "WINNER",
                          JOptionPane.DEFAULT_OPTION,
                          JOptionPane.INFORMATION_MESSAGE);
        }

        if (display instanceof ServerBattle) {
            ((ServerBattle) display).gameOver();
        }
    }
    
    /**
     * Process the network command
     * @param cmd the command to process
     */
    public void processCmd(String cmd) {
        StringTokenizer tok = new StringTokenizer(cmd, ",");
        int id = Integer.parseInt(tok.nextToken());
        String type = tok.nextToken();
        if (type.equals("infect")) { //infect
            int oid = Integer.parseInt(tok.nextToken());
            creatures[oid].setAction(2);
            creatures[oid].setNewSpecie(creatures[id].getCurrentSpecie());
        } else if (type.equals("turn")) { //turn
            int newdir = Integer.parseInt(tok.nextToken());
            creatures[id].setAction(3);
            creatures[id].setNewDirection((float) newdir);
        } else if (type.equals("hop")) { // hop command
            creatures[id].setAction(1);
        } else if (type.equals("winner")) { 
            playing = false;
            winner = tok.nextToken();
            emptyQueue();
        } else { //unknown type so just skip
            readyNext = true;
        }
        
    }
    /**
     * Stores received cmds into a queue until we are ready for
     * them
     * @param cmd The cmd to preformed
     */
    public void addToQueue(String cmd) {
        synchronized (cmdQueue) {
            cmdQueue.add(cmd);
        }
    }
    
    /**
     * This is set by MainClass when they get the cmd data 
     * from the server
     * @param serverWait The serverWait to set.
     */
    public void setServerWait(boolean serverWait) {
        this.serverWait = serverWait;
    }
    
    
    /**
     * This is set by the creature when they are done moving
     * @param readyNext The readyNext to set.
     */
    public void setReadyNext(boolean readyNext) {
        this.readyNext = readyNext;

    }
    
    /**
     * This emptys the queue of remaining commands
     */
    public void emptyQueue() {
        synchronized (cmdQueue) {
            while (!cmdQueue.isEmpty()) {
                cmdQueue.remove(0);
            }
        }
    }
    
    /**
     * @param off The xOff to set.
     */
    public void setXOff(int off) {
        xOff = off;
    }
    /**
     * @param off The yOff to set.
     */
    public void setYOff(int off) {
        yOff = off;
    }
    
    /**
     * @return Returns the rotate.
     */
    public boolean isRotate() {
        return rotate;
    }
    /**
     * @param rotate The rotate to set.
     */
    public void setRotate(boolean rotate) {
        this.rotate = rotate;
    }
}
