package edu.cs2335.lab7.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.net.Socket;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import edu.cs2335.lab7.server.GameData;

/**
 * 
 */
public class ClientConnection implements Runnable {

    /**Reference to the client main class.*/
    private MainClass main;
    /**PrintWriter to send strings over the network.*/
    private PrintWriter output;
    /**BufferedREader to read messages from the network.*/
    private BufferedReader input;
    /**Input stream.*/
    private InputStreamReader instream;
    /**Input stream to read objects from server.*/
    private ObjectInputStream objectInput;
    /**Output stream to send objects to the server.*/
    private ObjectOutputStream objectOutput;
    /**The connection.*/
    private Socket clientsocket;
    /**Controls whether execution of the thread continues.*/
    private boolean running;
    /**This user's username.*/
    private String username;
    /**Game test*/
    private boolean gamestate;
    
    /**
     * Constructor that sets up the thread.
     * @param mainclass The control class
     * @param hostname host to connect to
     * @param port the port to connect to
     * @throws IOException thrown if errors
     */
    public ClientConnection(MainClass mainclass, String hostname, String port) 
        throws IOException {
            running = true;
            main = mainclass;
            clientsocket =  new Socket(hostname, Integer.parseInt(port));
            output = new PrintWriter(clientsocket.getOutputStream());
            instream = new InputStreamReader(clientsocket.getInputStream());
            input = new BufferedReader(instream);
            gamestate = false;
    }
    
    /**
     * Main loop. Receives messages from the server.
     */
    public void run() {
        String messagefromserver = null;
        String[] message = null;
        try {
            while (running) {
                messagefromserver = input.readLine();
                if (messagefromserver == null) {
                    continue;
                }
                message = messagefromserver.split("~");
                parseMessage(message);
            }
        } catch (IOException ioe) {
            running = false;
            final IOException e = ioe;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    e.printStackTrace();
                    main.displayError("SF", "connecterror");
                }
            });
        }
    }
    
    /**
     * Parses the message read by the server and calls an appropriate method
     * @param text  an array containing the tokens of the message
     */
    private void parseMessage(String[] text) {
        if (text[0].equals("GAMEMOVE")) {
            processMoves(text);
        } else if (text[0].equals("ACTIVEUSERS")) {
            main.updateActiveUsers(text);
        } else if (text[0].equals("ACTIVEGAMES")) {
            main.updateActiveGames(text);
        } else if (text[0].equals("GAMEUSERS")) {
            main.updateGameUsers(text);
        } else if (text[0].equals("CHALLENGE")) {
            displayChallenge(text);
        } else if (text[0].equals("USERDECL")) {
            displayMessage(text);
        } else if (text[0].equals("USERSTAT")) {
            displayChallengeConfirm(text);
        } else if (text[0].equals("CREATLIST")) {
            main.displayCreatureList(text);
        } else if (text[0].equals("MAINMSG")) {
            main.displayMainChatMessage(text);
        } else if (text[0].equals("GAMEMSG")) {
            main.displayGameChatMessage(text);
        } else if (text[0].equals("CHOOSEOPTIONS")) {
            chooseGameOptions(text);
        } else if (text[0].equals("CHOOSECREAT")) {
            chooseGameCreature(text);
        } else if (text[0].equals("STARTGAME")) {
            receiveGameData();
        } else if (text[0].equals("GAMEMOVES")) {
            processMoves(text);
        } else if (text[0].equals("CANTCHAL")) {
            displayErrorMessage(text);
        } else if (text[0].equals("CANTJOIN")) {
            displayErrorMessage(text);
        } else if (text[0].equals("NOCREAT")) {
            displayErrorMessage(text);
        } else if (text[0].equals("SF")) {
            shutdownClient();
            displayShutdownMessage();
        } else if (text[0].equals("CS")) {
            sendCreature();
        } else if (text[0].equals("RS")) {
            displayMessage(text);
        } else if (text[0].equals("LS")) {
            loginUser();
        } else if (text[0].equals("X")) {
            displayErrorMessage(text);
        } else {
            displayErrorMessage(text);
        }
    }
    
    /**
     * use this to send a message to the server
     * @param msg  the message to send 
     */
    public void sendMessage(String msg) {
        output.println(msg);
        output.flush();
        System.out.println(msg);
    }
    
    
    /**
     * Uploads a creature to the server.
     */
    public void sendCreature() {
        try {
            if (objectOutput == null) {
            objectOutput = new ObjectOutputStream(clientsocket
                    .getOutputStream());
            }
            objectOutput.writeObject(main.getNewCreature());
            objectOutput.flush();
          } catch (IOException e) {
              e.printStackTrace();
            main.displayError("error", "There was an error uploading " 
                    + "the creature.");
        }
    }
    
    
    /**
     * Sent to challengee. Displays options for selecting game parameters.
     * @param options  maplist and creaturelist
     */
    public void chooseGameOptions(String[] options) {
        final String[] optionlist = options;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                main.chooseOptions(optionlist);
            }
        });
    }
    
    
    /**
     * Sent to challenger. Displays options for selecting the creature.
     * @param options  creaturelist
     */
    public void chooseGameCreature(String[] options) {
        final String[] optionlist = options;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                main.chooseCreature(optionlist);
            }
        });
    }
    
    /**
     * Recieves the gamedata object from the server to set up the game.
     */
    public void receiveGameData() {
        gamestate = true;
        GameData gameData = null;

        // wait to get game data
        try {
            if (objectInput == null) {
            objectInput = new ObjectInputStream(clientsocket.getInputStream());
            }
            gameData = (GameData) objectInput.readObject();
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Client gamedata exception");
            return;
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            System.out.println("Client gamedata exception");
            return;
        }
        if (gameData == null) {
            return;  // error
        }

        // setup a new battle
        final GameData data = gameData;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                main.createBattle();
                main.getBattle().initBattle(data);
                sendMessage("READY");
            }
        });
        
    }
    
    /**
     * Process a list of creature moves.
     * @param moves  a list of moves
     */
    public void processMoves(String[] moves) {
        main.battleCmds(moves);
    }

    /**
     * Displays a message when this client receives a challenge.
     * @param message  the challenge message
     */
    private void displayChallengeConfirm(String[] message) {
        final String msg = "Are you sure you want to challenge " + message[1]
        + "?\nUser record: Wins - " + message[2] + "  Losses - " + message[3];
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (JOptionPane.
                        showConfirmDialog(main.getLobby(),
                                          msg,
                                          "Notice",
                                          JOptionPane.YES_NO_OPTION,
                                          JOptionPane.INFORMATION_MESSAGE)
                        == JOptionPane.YES_OPTION) {
                        sendMessage("CHALLENGE~" + main.getSelectedUser());
                    }
            }
        }); 
    }
    
    /**
     * This display the challenge message
     * @param message the message sent from the challenger
     */
    public void displayChallenge(String[] message) {
        final String challenginguser = message[1];
        final String msg = challenginguser + " has challenged you.\n"
        + "Do you with to accept?\n"
        + "User record: Wins - " + message[2] + "  Losses - " + message[3];
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (JOptionPane.showConfirmDialog(main.getLobby(),
                        msg, "Challenge Received", JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE)
                        == JOptionPane.YES_OPTION) {
                        sendMessage(
                               "ACPTCHAL~" + username + "~" + challenginguser);
                    } else {
                        sendMessage(
                                "DECLCHAL~" + challenginguser);
                    }
            }
        }); 
    }
  

    /**
     * Displays a message to the user when an error occurs.
     * @param message  a message describing the error
     */
    private void displayErrorMessage(String[] message) {
        String errormessage = "";
        String errorcode = "";
        if (message.length > 1) {
        errorcode = message[1];
        }
        if (message.length > 2) {
        errormessage = message[2];
        }
        String title = "";
        String displaymessage = null;
        System.out.println(errorcode);
        System.out.println(errormessage);
        if (errorcode.equals("RF")) {
            title = "Registration Failure";
            displaymessage = "That username has already been registered.";
        } else if (errorcode.equals("CF")) {
            title = "Request Failure";
            if (errormessage.equals("creatureexists")) {
                displaymessage = "A creature with that name already exists.";
            }
        }  else if (errorcode.equals("LF")) {
            title = "Login Failure";
            if (errormessage.equals("unknownuser")) {
                displaymessage = "Username not valid. Please try again.";
            } else if (errormessage.equals("badpassword")) {
                displaymessage = "You supplied an incorrect password.";
            }
        } else if (errorcode.equals("MF")) {
            title = "Message Failure";
            displaymessage = "User is offline.";
        } else if (message[0].equals("CANTCHAL")) {
            displaymessage = "Challenge request failed.\n"
                           + "You are already watching or playing a game.";
        } else if (message[0].equals("CANTJOIN")) {
            displaymessage = "Join request failed.\n"
                           + "You are already watching or playing a game.";
        } else if (message[0].equals("NOCREAT")) {
            displaymessage = "Request failed.\n"
                           + "Either you or the person you challenged\nhas not "
                           + "uploaded a creature to the server.";
        } else {
            return;
        }
        final String titl = title;
        final String msg = displaymessage;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                main.displayError(titl, msg);
            }
        }); 
    }
    
    /**
     * Display the shutdown message 
     */
    private void displayShutdownMessage() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
        String title = "Server Failure";
        String displaymessage = "The server shutdown. You have "
                + "been logged off.";
        if (JOptionPane.
                showConfirmDialog(null,
                                  displaymessage,
                                  title,
                                  JOptionPane.DEFAULT_OPTION,
                                  JOptionPane.INFORMATION_MESSAGE)
                == JOptionPane.YES_OPTION) {
                System.exit(0);
            }
            }
        });
    }
    
    /**
     * Displays a notice to the user that something occurred (nonerror).
     * @param message  the message to display
     */
    private void displayMessage(String[] message) {
        String sentmessage = null;
        if (message[0].equals("RS")) {
            sentmessage = "Registration successful.\n" + "You may now login.";
        } else if (message[0].equals("USERDECL")) {
            sentmessage = message[1] + " refused your challenge.";
        } final String msg = sentmessage;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                main.displayMessage(msg);
            }
        }); 
    }
    
    /**
     * Logs the user into the main window.
     */
    private void loginUser() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                main.switchWindow(Conts.LOBBY_WIN);
            }
        }); 
    }

    /** Shuts down this thread and closes connections */
    public synchronized void shutdownClient() {
        try {
            if (gamestate) {
                sendMessage("LEAVEGAME");
            }
            sendMessage("QUIT");
            main.closeWindows();
            running = false;
            input.close();
            output.close();
            instream.close();
            clientsocket.close();
        } catch (IOException e) {
            e.printStackTrace();
            running = false;
            return;
        }
    }
    
    /**
     * @return Returns the username.
     */
    public String getUsername() {
        return username;
    }
    
    /**
     * @param username The username to set.
     */
    public void setUsername(String username) {
        this.username = username;
    }

    /**
     * @return Returns the client's socket.
     */
    public Socket getClientSocket() {
        return clientsocket;
    }
}
