
package edu.cs2335.lab7.client;

import java.awt.Graphics2D;
import java.awt.Image;
import java.io.Serializable;

import edu.cs2335.lab7.creature.DWSpecies;
import edu.cs2335.lab7.data.CreatureData;

/**
 * @author thaile
 *
 * Animates and draws the creatures 
 */
public class CreatureRender implements Serializable {
    /** CONT: cmds constant hop */
    private static final int HOP = 1;
    /** CONT: cmds constant hop */
    private static final int INFECT = 2;
    /** CONT: cmds constant hop */
    private static final int TURN = 3;
    /** CONT: indicating a north facing creature */
    public static final int NORTH = 0;
    /** CONT: indicating a south facing creature */
    public static final int SOUTH = 180;
    /** CONT: indicating an east facing creature */
    public static final int EAST = 90;
    /** CONT: indicating a west facing creature */
    public static final int WEST = 270;
    
    /** DATA: image of the direction the creature is facing */
    private Image dirTile = null;
    /** DATA: turn speed 45 degrees per seconds */
    private static final int TURNSPEED = 180;
    /** DATA: hopping speed 16 pixels per seconds */
    private static final int HOPSPEED = 64;
    
    /** Reference to the battle */
    private Battle battle;
    // New and current are for animation perpose
    
    /** DATA: new creature species */
    private DWSpecies newSpecie;
    /** DATA: new x location */
    private float newX;
    /** DATA: new y location */
    private float newY;
    /** DATA: new direction */
    private float newDirection;
    
    /** DATA: current creature species */
    private DWSpecies currentSpecie;
    /** DATA: current x location */
    private float currentX;
    /** DATA: current y location */
    private float currentY;
    /** DATA: current direction */
    private float currentDirection;
    
    /** FLAG: Action to preformed */
    private int action; 
    /** FLAG: direction to turn */
    private boolean turnRight = false;
    
    /**
     * Creates a new creature to be rendered
     * @param game The game the creature being rendered on
     * @param specie the specie type
     * @param data the creature info
     */
    public CreatureRender(Battle game, DWSpecies specie, CreatureData data) {
        this.battle = game;
        this.currentSpecie = specie;
        this.newSpecie = specie;
        this.currentX = data.getX() * 32;
        this.newX = data.getX() * 32;
        this.currentY = data.getY() * 32;
        this.newX = data.getX() * 32;
        this.currentDirection = data.getDirection();
        this.newDirection = data.getDirection();
        action = 0;
        
    }
    
    
    /**
     * @param dirTile the direction the creature is facing
     */
    public void setDirTile(Image dirTile) {
        this.dirTile = dirTile;
    }
    
    /**
     * 
     * This moves the creature base on time between moves
     * and a couple of other things
     * @param time the time since the last render command
     */   
     public void move(long time) {
       if (action == HOP) {         
            float disMoved = (time * HOPSPEED) / 1000;
            if (currentDirection == NORTH) {
                currentY -= disMoved;
                if (currentY <= newY) {
                    currentY = newY;
                    action = 0;
                    battle.setReadyNext(true);
                }
            } else if (currentDirection == WEST) {
                currentX -= disMoved;
                if (currentX <= newX) {
                    currentX = newX;
                    action = 0;
                    battle.setReadyNext(true);
                }
            } else if (currentDirection == SOUTH) {
                currentY += disMoved;
                if (currentY >= newY) {
                    currentY = newY;
                    action = 0;
                    battle.setReadyNext(true);
                }
            } else if (currentDirection == EAST) {
                currentX += disMoved;
                if (currentX >= newX) {
                    currentX = newX;
                    action = 0;
                    battle.setReadyNext(true);
                }
            } else {
                System.out.println("something wrong");
            }
       } else if (action == INFECT) {
            currentSpecie = newSpecie;
            battle.setReadyNext(true);
       } else if (action == TURN) {
            float disTurned = (time * TURNSPEED) / 1000;
            if (turnRight) {
                currentDirection += disTurned;
                if (currentDirection >= newDirection) {
                    currentDirection = newDirection;
                    turnRight = false;
                    action = 0;
                    battle.setReadyNext(true);
                }
            } else {
                currentDirection -= disTurned;
                if (currentDirection <= newDirection) {
                    currentDirection = newDirection;
                    action = 0;
                    battle.setReadyNext(true);
                }
            }
       } 
    }
    
    /**
     * Actually draws the picture to screen
     * @param g what to draw on
     * @param xOff offset on the x-axis
     * @param yOff offset on the y-axis
     */
    public void render(Graphics2D g, int xOff, int yOff) {
        //rotate the creature if needed
        int x = (int) currentX - xOff;
        int y = (int) currentY - yOff;
        g.translate(x + 16, y + 16);
        
        float iconShift = currentDirection - 90;
        if (iconShift < 0) {
            iconShift += 360;
        }
        
        g.rotate(iconShift * Math.PI / 180);
   
        //draw the creature
        g.drawImage(currentSpecie.getIcon().getImage(), -16, -16, null);
        
        //draw the direction indecator image if present
        
        if (dirTile != null) {
            g.rotate(90 * Math.PI / 180);
            g.drawImage(dirTile, -16, -16, null);
            g.rotate(-(90 * Math.PI / 180));
        }
        
        g.rotate(-(iconShift * Math.PI / 180));
        g.translate(-(x + 16), -(y + 16));
    } 
        
    /**
     * @param action The action to set.
     */
    public void setAction(int action) {
        this.action = action;
        if (action == HOP) {
            switch((int) currentDirection) {
            case NORTH:
                newY = currentY - 32;
                break;
            case EAST:
                newX = currentX + 32;
                break;
            case SOUTH:
                newY = currentY + 32;
                break;
            case WEST:
                newX = currentX - 32;
                break;
            default:
            }
        }
        
        
    }
    /**
     * @param newDirection The newDirection to set.
     */
    public void setNewDirection(float newDirection) {
        this.newDirection = newDirection;
        if ((currentDirection + 90) == newDirection) {
            turnRight = true;
        } else if ((currentDirection + 90) == 360 && newDirection == 0) {
            turnRight = true;
        } else {
            if (currentDirection == 0) {
                currentDirection = 360;
            }
        }
    }
    /**
     * @param newSpecie The newSpecie to set.
     */
    public void setNewSpecie(DWSpecies newSpecie) {
        this.newSpecie = newSpecie;
    }
    
    /**
     * @return Returns the currentSpecie.
     */
    public DWSpecies getCurrentSpecie() {
        return currentSpecie;
    }
}
