package edu.cs2335.lab7.client;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

/**
 * This is an generic unzipping program it takes in a file and where to extect
 * the file to.
 * 
 * @author thaile
 *  
 */
public class DataUnzip implements Serializable {

    /**
     * This is for checkstyle and PMD enjoyment
     */
    private DataUnzip() {

    }

    /**
     * This extracts data from a zip file
     * 
     * @param file
     *            The zip file to extract
     * @param extractTo
     *            where to extract to.
     */
    public static void unzip(final File file, final String extractTo) {
        Enumeration entries;
        ZipFile zipfile = null;

        try {
            //create the destanation directory
            File desDir = new File(extractTo);
            desDir.mkdir();
            
            zipfile = new ZipFile(file);
            entries = zipfile.entries();

            while (entries.hasMoreElements()) {
                final ZipEntry entry = (ZipEntry) entries.nextElement();
                //this is special for Species file
                if ((entry.getName().substring(0, 3)).equals("img")) {
                    File temp = new File(entry.getName());
                    File tempDir = new File(extractTo + "/"  
                            + temp.getParent());
                    tempDir.mkdir();
                    copyInputStream(zipfile.getInputStream(entry), 
                            extractTo + "/" + tempDir.getName(), 
                            entry.getName().substring(4));
                    
                } else {
                    copyInputStream(zipfile.getInputStream(entry), extractTo, 
                            entry.getName());
                }

            }
            
            zipfile.close();
        } catch (ZipException e) {
            System.out.println("Something wrong with the zip file");
            System.exit(0);
        } catch (IOException e) {
            System.out.println("Couldn't write the file");
            System.exit(0);
        }

    }

    /**
     * This takes in an inputstream from the zipfile and writes it to the file
     * 
     * @param input
     *            The input Stream for the zipfile
     * @param extractTo
     *            where to write the files
     * @param name
     *            The name of the file to write to
     * @throws IOException
     *             Couldn't write the file
     */
    public static void copyInputStream(final InputStream input,
            final String extractTo, final String name) throws IOException {
        byte[] buffer = new byte[1024 * 6];
        int len;
        File temp = new File(extractTo, name);
        FileOutputStream theFile = new FileOutputStream(temp);
        BufferedOutputStream extractFile = new BufferedOutputStream(
                theFile);

        while ((len = input.read(buffer)) >= 0) {
            extractFile.write(buffer, 0, len);
        }

        input.close();
        extractFile.close();
    }
    

}