
package edu.cs2335.lab7.client;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

/**
 */
public class LobbyWindow extends JFrame {

    /**The box where the user types in his/her chat.*/
    private JTextArea inputText;
    /**The area that displays the connected users.*/
    private JPanel userpane;
    /**The area on the west side of the screen */
    private JPanel west;
    /**This is primary user display part of the window.*/
    private JScrollPane userdisplay;
    /**A list of connected users..*/
    private JList users;
    /**The area that displays the connected games.*/
    private JList games;
    /**This is primary game display part of the window.*/
    private JScrollPane gamedisplay;
    /**Main Chat window.*/
    private JTextArea chatpane;
    /**The button that sends what is in the inputText box.*/
    private JButton send;
    /**The username of the user.*/
    private String username;
    /**The options menu.*/
    private MainClass main;
    /** The connection to the server */
    private ClientConnection connection;
    /** The stable of creatures for the user */
    private Stable stable;
    /** The scrnheight */
    private int scrnheight;
    /** The scrnwidth */
    private int scrnwidth;
    
    /**
     * Creates and sets up the JFrame and everything inside of it.
     * @param main reference to controling class
     * @param conn the client connection
     * @param screenheight int - the height of the screen
     * @param screenwidth int - the width of the screen
     */
    public LobbyWindow(MainClass main, ClientConnection conn, int screenwidth,
            int screenheight) {
        super();
        this.main = main;
        connection = conn;
        scrnheight = screenheight;
        scrnwidth = screenwidth;
        username = main.getConnection().getUsername();
        setTitle("Darwin's World 1.0 - Connected as " + username);
        this.setResizable(false);
        this.getContentPane().setLayout(new BorderLayout());
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeWindow();
            }
        });
        JMenuBar menu = createMenuBar();
        JPanel mainPane = (JPanel) this.getContentPane();
        mainPane.setPreferredSize(new Dimension(800, 600));
        setSize(800, 600);
        JPanel east = createEast();
        JPanel westSide = createWest();
        mainPane.add(east, BorderLayout.EAST);
        mainPane.add(westSide, BorderLayout.WEST);
        mainPane.add(menu, BorderLayout.NORTH);
        pack();
        setLocation((screenwidth - 800) / 2, (screenheight - 600) / 2);
        setVisible(true);
    }
    /**
     * Creates the menubar for the top.
     * @return JMenuBar - the create menu bar
     */
    private JMenuBar createMenuBar() {
        JMenuBar optionsBar = new JMenuBar();
        JMenu options = new JMenu("Options");
        JMenuItem invite = new JMenuItem("Manage Stable");
        options.add(invite);
        invite.addActionListener(new StableMenuListener());
        options.addSeparator();
        JMenuItem close = new JMenuItem("Quit");
        close.addActionListener(new CloseMenuListener());
        close.addActionListener(new SendListener());
        options.add(close);
        optionsBar.add(options);
        return optionsBar;
    }
    /**
     * Creates the list of people talking
     * @return JPanel - the created panel with everything on it
     */
    private JPanel createWest() {
        west = new JPanel();
        
        JPanel southwest = new JPanel();
        southwest.setPreferredSize(new Dimension(580, 70));
        southwest.setLayout(new BoxLayout(southwest, BoxLayout.X_AXIS));
        southwest.setBorder(new EmptyBorder(6, 5, 6, 6));
        JPanel titlepanel = new TitlePanel();
        inputText = new JTextArea();
        inputText.setLineWrap(true);
        inputText.setPreferredSize(new Dimension(450, 60));
        inputText.setBorder(BorderFactory.createLineBorder(Color.black));
        southwest.add(inputText);
        send = new JButton("Send");
        send.setPreferredSize(new Dimension(90, 30));
        send.addActionListener(new SendListener());
        southwest.add(Box.createRigidArea(new Dimension(10, 2)));
        southwest.add(send);
        southwest.add(Box.createRigidArea(new Dimension(5, 2)));

        JPanel northwest = new JPanel();
        chatpane = new JTextArea();
        chatpane.setEditable(false);
        chatpane.setLineWrap(true);
        JScrollPane chatdisplay = new JScrollPane(chatpane);
        chatdisplay.setPreferredSize(new Dimension(600, 500));
        northwest.add(chatdisplay);
        
        west.setLayout(new BorderLayout());
        west.add(titlepanel, BorderLayout.NORTH);
        west.add(southwest, BorderLayout.SOUTH);
        west.add(northwest, BorderLayout.CENTER);
        return west;
    }
    /**
     * Creates the center panel to display the chat text.
     * @return JPanel - the panel with everything added to it
     */
    private JPanel createEast() {
        JPanel east = new JPanel();
        JPanel northeast = new JPanel();
        JPanel southeast = new JPanel();
        northeast.setPreferredSize(new Dimension(200, 250));
        southeast.setPreferredSize(new Dimension(200, 250));
        east.setLayout(new BoxLayout(east, BoxLayout.Y_AXIS));
        northeast.setLayout(new BoxLayout(northeast, BoxLayout.Y_AXIS));
        southeast.setLayout(new BoxLayout(southeast, BoxLayout.Y_AXIS));
        userpane = new JPanel();
        users = new JList();
        userdisplay = new JScrollPane(users);
        userdisplay.setPreferredSize(new Dimension(150, 200));
        userdisplay.setBorder(new TitledBorder("Online Users:"));
        userpane.add(userdisplay);
        northeast.add(userpane);
        
        JPanel challenge = new JPanel();
        JButton challengebutton = new JButton ("Challenge");
        challengebutton.addActionListener(new ChallengeListener());
        challenge.setPreferredSize(new Dimension(130, 30));
        challenge.add(challengebutton);
        northeast.add(challenge);
        
        JPanel gamespane = new JPanel();
        games = new JList();

        gamedisplay = new JScrollPane(games);
        gamedisplay.setPreferredSize(new Dimension(150, 200));
        gamedisplay.setBorder(new TitledBorder("Active Games:"));
        gamespane.add(gamedisplay);
        southeast.add(gamespane);
        
        JPanel watchgame = new JPanel();
        JButton watchgamebutton = new JButton ("Watch Game");
        watchgamebutton.addActionListener(new WatchListener());
        watchgame.setPreferredSize(new Dimension(130, 30));
        watchgame.add(watchgamebutton);
        southeast.add(watchgame);
        
        east.add(Box.createRigidArea(new Dimension(180, 10)));
        east.add(northeast);
        east.add(Box.createRigidArea(new Dimension(180, 20)));
        east.add(southeast);
        east.add(Box.createRigidArea(new Dimension(180, 20)));
        
        return east;
    }
    
    /**
     * Returns the text entered into the typing box and clears it.
     * @return String - the text entered into the typing box
     */
    public String getChatText() {
        String temp = inputText.getText();
        inputText.setText("");
        return temp;
    }
    
    /**
     * Adds a user to the Conference.
     * @param userlist String - the user to add to the conference
     */
    public final void updateUsers(String[] userlist) {
        String[] listdata = null;
        if (userlist.length < 3) {
            listdata = new String[1];
            listdata[0] = "No active users.";
        } else {
            listdata = new String[userlist.length - 2];
            int newindex = 0;
            for (int i = 1; i < userlist.length - 1; i++) {
                listdata[newindex] = userlist[i];
                newindex++;
            }
        }
           users.setListData(listdata);
           users.repaint();
    }
    
    /**
     * Updates the game list 
     * @param gamelist the list of active games
     */
    public final void updateGames(String[] gamelist) {
        String[] listdata = null;
        if (gamelist.length < 3) {
            listdata = new String[1];
            //listdata[0] = "No active games.";
        } else {
            listdata = new String[gamelist.length - 2];
            int newindex = 0;
            for (int i = 1; i < gamelist.length - 1; i++) {
                listdata[newindex] = gamelist[i];
                newindex++;
            }
        }
           games.setListData(listdata);
           games.repaint();
    }

    /**
     * Returns the name of the chosen user
     * @return Username
     */
    public String getUserChosen() {
        return (String) (users.getSelectedValue());
    }
    
    /**
     * Adds the text to the display window.
     * @param input String - the text to add to the window.
     */
    public void addText(String[] input) {
            if (!chatpane.equals("")) {
                chatpane.append("\n");
            }
            chatpane.append(input[1] + ": " + input[2]);
            chatpane.setCaretPosition((chatpane.getDocument()).getLength() - 1);
    }
     
    /**
     * Closes the active window.
     */
    public void closeWindow() {
        if (stable != null) {
            stable.dispose();
        }
            (main.getConnection()).shutdownClient();
        System.exit(0);
    }
    
    /**
     * Closes the window on quit
     */
    public void closeOnQuit() {
        if (stable != null) {
            stable.dispose();
        }
    }
    
    /**
     * set the stable to null?
     */
    public void closeStable() {
        stable = null;
    }
    
    /**
     * Enables/Disables the JFrame.
     * @param enabled boolean - determines to enable or diable JFrame
     */
    public void setWindowEnabled(boolean enabled) {
        this.setEnabled(enabled);
    }
    
    /** 
     * This is what creates a title panel
     */
    private class TitlePanel extends JPanel {
        /** The font size to use */
        private int fontsize;
        
        /**
         * Inits the JPanel settings
         */
        public TitlePanel() {
        setPreferredSize(new Dimension(400, 50));
        setForeground(Color.black);
        fontsize = 28;
        }
        
        /** 
         * Draws the panel
         * @param g the graphic to draw it on
         */ 
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            String title = "Darwin's World Main Lobby";
            Font font = new Font("Sans", Font.BOLD, fontsize);
            g.setFont(font);

            FontMetrics tempmet   = g.getFontMetrics(font);  
            Rectangle2D rect = tempmet.getStringBounds(title, g);

            int textHeight = (int) (rect.getHeight());
            int textWidth  = (int) (rect.getWidth());
            int panelHeight = this.getHeight();
            int panelWidth = this.getWidth();

            int x = (panelWidth  - textWidth)  / 2;
            int y = (panelHeight - textHeight) / 2  + tempmet.getAscent();

            g.drawString(title, x, y);
        }
    }
    
    
    /**Is called when the user selects manage stable from the menu.*/
    private class StableMenuListener implements ActionListener {
        /**
         * Opens the stable manager window.
         * @param e ActionEvent - the source (Invite)
         */
        public void actionPerformed(ActionEvent e) {
            if (stable == null) {
            stable = new Stable(main, scrnwidth, scrnheight);
            connection.sendMessage("LISTCREAT");
            } else {
                stable.requestFocus();
            }
        }
       }
        

    /**Is called when the user selects quit from the menu.*/
    private class CloseMenuListener implements ActionListener {
        /**
         * Closes the window and closes the connection.
         * @param e ActionEvent - the source (Close)
         */
        public void actionPerformed(ActionEvent e) {
            closeWindow();
        }
    }
    /**Is called when the user sends a message.*/
    private class SendListener implements ActionListener {
        
        /**
          * This ActionEvent sends the text to the server.
          * @param e ActionEvent - the source of the call (Send)
          */
         public void actionPerformed(ActionEvent e) {
             String message = getChatText();
             if (message != null && message != "") {
                (main.getConnection()).sendMessage("MAINMSG~" + message);
             }
         }
     }
    /**Is called when the user challenge someone */
    private class ChallengeListener implements ActionListener {
        /** The action performed
         * @param e the actionEvent
         */
        public void actionPerformed(ActionEvent e) {
            String value = (String) (users.getSelectedValue());
            if (value != null && !(value.equals(username))) {
                connection.sendMessage("GETUSERSTAT~" 
                        + (String) users.getSelectedValue());
            }
        }
    }
    /**Is called when the user wants to watch a battle */
    private class WatchListener implements ActionListener {
        /**
         * Sends a joun game msg to the server
         * @param e the actionevent
         */
        public void actionPerformed(ActionEvent e) {
            String value = (String) (games.getSelectedValue());
            if (value != null && !(value.equals(username))
                    && !(value.equals("No active games."))) {
                connection.sendMessage("JOINGAME~" + value);
            }
        }
    }
    
        
    
    /**
     * @return Returns the stable.
     */
    public Stable getStable() {
        return stable;
    }
}