/*
 * Created on Mar 9, 2005
 *
 * The Local Battle GUI.
 */
package edu.cs2335.lab7.client;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import edu.cs2335.lab7.creature.DWSpecies;
import edu.cs2335.lab7.data.MapData;
import edu.cs2335.lab7.data.MapMaker;
import edu.cs2335.lab7.data.SpeciesMaker;
import edu.cs2335.lab7.server.Game;

/**
 * @author thaile
 * Creates a new game locally by pretending to be the server.  
 */
public class LocalBattle extends JFrame {

    /** The Canvas for drawing */
    private Battle mapDisplay;
    /** The link to MainClass */
    private MainClass main;
    
    /** Horizontal scroll bar */
    private JScrollBar hScroll;
    /** Vertical scroll bar */
    private JScrollBar vScroll;

    
    
    // *** Game Setting control *** //
    /** Display map selected */
    private JLabel mapLabel;
    /** Display name of species selected */
    private JLabel species1Label;
    /** Display the creature icon */
    private JLabel species1Icon;
    /** Display name of species selected */
    private JLabel species2Label;
    /** Display the creature icon */
    private JLabel species2Icon;
    /** This creates a new game */
    private JButton createButton;
    /** This ends the game */
    private JButton endButton;
    /** ButtonGroup for number of creatures */
    private ButtonGroup numGroup;
    
    
    // *** Game Flow control *** //
    /** This is the number of steps to take */
    private JTextField numSteps;
    /** This is the pause button */
    private JButton pauseButton;
    /** This is the play button */
    private JButton playButton;
    /** This is the Next button */
    private JButton nextButton;
    /** This is the next n button */
    private JButton nextNButton;
    
    // *** Game data *** //
    /** This holds the game to run */
    private Game game;
    /** This holds the first species object */
    private DWSpecies species1 = null;
    /** his holds the second species object */
    private DWSpecies species2 = null;
    /** This holds the map data */
    private MapData map = null;
    /** This holds the number of creatures they want */
    private int numCreatures = 5;
    /** This is what called when play is clicked */
    private PlayGame theGame = null;
    /** FLAG: the game is ending NOW!! */
    private boolean endNow = false;
    /**
     * Starts up a local battle gui and controls the game
     * @param main the link to the mainClass
     * @param screenwidth the screenwidth
     * @param screenheight the screenheight
     */
    public LocalBattle(MainClass main, int screenwidth, int screenheight) {
        super("L O C A L    B A T T L E  - darwin world");
        this.main = main;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeThis();
                
             }
        });
        JPanel mainPane = new JPanel();
        mainPane.setPreferredSize(new Dimension(800, 600));
        mainPane.setLayout(new BorderLayout());
        mainPane.add(gameSetupPane(), BorderLayout.WEST);
        mainPane.add(gameFlowPane(), BorderLayout.SOUTH);
        mainPane.add(mapPane(), BorderLayout.CENTER);
        getContentPane().add(mainPane);
        setSize(new Dimension(800, 600));
        setResizable(false);
        setLocation((screenwidth - 800) / 2, (screenheight - 600) / 2);
        pack();
        show();
        
    }
    
    /**
     * This calls the switch window on the main
     */
    public void callSwitch() {
        main.switchWindow(Conts.STARTUP_WIN);
    }
    /** 
     * This creates the panel with all the setup setting for 
     * the local battle
     * @return A JPanel
     */
    public final JPanel gameSetupPane() {
        JPanel pane = new JPanel();
        
        pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
        
        JPanel temp = new JPanel();
        temp.setBorder(BorderFactory.createTitledBorder("M A P"));
        
        mapLabel = new JLabel("<Select>");
        JButton findMapButton = new JButton("Browse");
        findMapButton.setActionCommand("MAP");
        findMapButton.addActionListener(new FileFinder());
        
        temp.add(mapLabel);
        temp.add(findMapButton);
        pane.add(temp);
        
        temp = new JPanel();
        temp.setBorder(BorderFactory.createTitledBorder("Number of Creatures"));
        
        JPanel temp2 = new JPanel();
        temp2.setLayout(new BoxLayout(temp2, BoxLayout.Y_AXIS));
        JRadioButton num1 = new JRadioButton("1 vs 1");
        JRadioButton num5 = new JRadioButton("5 vs 5");
        JRadioButton num10 = new JRadioButton("10 vs 10");
        JRadioButton num20 = new JRadioButton("20 vs 20");
        
        num1.setActionCommand("1");
        num5.setActionCommand("5");
        num10.setActionCommand("10");
        num20.setActionCommand("20");
        
        num1.addActionListener(new NumCreatureChanger());
        num5.addActionListener(new NumCreatureChanger());
        num10.addActionListener(new NumCreatureChanger());
        num20.addActionListener(new NumCreatureChanger());
        
        num5.setSelected(true);
        
        numGroup = new ButtonGroup();
        numGroup.add(num1);
        numGroup.add(num5);
        numGroup.add(num10);
        numGroup.add(num20);
        
        temp2.add(num1);
        temp2.add(num5);
        temp2.add(num10);
        temp2.add(num20);
        temp.add(temp2);
        pane.add(temp);
        
        temp = new JPanel();
        temp.setBorder(BorderFactory.createTitledBorder("SPECIES 1"));
        species1Icon = new JLabel("");
        species1Label = new JLabel("<select>");
        JButton species1Button = new JButton("Browse");
        species1Button.setActionCommand("Species1");
        species1Button.addActionListener(new FileFinder());
        
        temp.add(species1Icon);
        temp.add(species1Label);
        temp.add(species1Button);
        pane.add(temp);
        
        temp = new JPanel();
        temp.setBorder(BorderFactory.createTitledBorder("SPECIES 2"));
        species2Icon = new JLabel("");
        species2Label = new JLabel("<select>");
        JButton species2Button = new JButton("Browse");
        species2Button.setActionCommand("Species2");
        species2Button.addActionListener(new FileFinder());
        
        temp.add(species2Icon);
        temp.add(species2Label);
        temp.add(species2Button);
        pane.add(temp);
        
        temp = new JPanel();
        temp.setLayout(new BoxLayout(temp, BoxLayout.Y_AXIS));
        temp.add(Box.createRigidArea(new Dimension(290, 0)));
        pane.add(temp);
        createButton = new JButton("Create Game");
        createButton.addActionListener(new CreateGame());
        endButton = new JButton("End Battle");
        endButton.addActionListener(new EndGame());
        pane.add(createButton);
        pane.add(endButton);
        return pane;
    }
    
    /**
     * This creates the panel for displaying the window
     * @return A JPanel
     */
    public final JPanel mapPane() {
        JPanel pane = new JPanel();
        mapDisplay = new Battle(this);
        mapDisplay.setBounds(0, 0, 480, 480);
        
        JPanel offSethScroll = new JPanel();
        offSethScroll.setLayout(new BoxLayout(offSethScroll, BoxLayout.X_AXIS));
        hScroll = new JScrollBar(JScrollBar.HORIZONTAL);
        hScroll.setPreferredSize(new Dimension(480, 20));
        hScroll.setEnabled(false);
        offSethScroll.add(hScroll);
        offSethScroll.add(Box.createRigidArea(new Dimension(20, 0)));

        vScroll = new JScrollBar(JScrollBar.VERTICAL);
        vScroll.setPreferredSize(new Dimension(20, 480));
        vScroll.setEnabled(false);
    
        TitledBorder tb = BorderFactory.createTitledBorder("map");
        pane.setBorder(tb);
        pane.setLayout(new BorderLayout());
        pane.setPreferredSize(new Dimension(500, 500));
        pane.add(mapDisplay, BorderLayout.CENTER);
        pane.add(offSethScroll, BorderLayout.SOUTH);
        pane.add(vScroll, BorderLayout.EAST);
        
        return pane;
    }
    
    /**
     * Set up the game control for the game
     * @return A JPanel
     */
    public final JPanel gameFlowPane() {
        URL url = this.getClass().getClassLoader().getResource(
        "images/pause.GIF");
        ImageIcon pauseI = new ImageIcon(url);
        url = this.getClass().getClassLoader().getResource(
        "images/play.GIF");
        ImageIcon playI = new ImageIcon(url);
        url = this.getClass().getClassLoader().getResource(
        "images/step.GIF");
        ImageIcon stepI = new ImageIcon(url);
        
        JPanel pane = new JPanel();
        pauseButton = new JButton(pauseI);
        playButton = new JButton(playI);
        nextButton = new JButton(stepI);
        nextNButton = new JButton(stepI);
        
        numSteps = new JTextField("5", 5);
        pauseButton.setActionCommand("pause");
        playButton.setActionCommand("play");
        nextButton.setActionCommand("next");
        nextNButton.setActionCommand("nextN");
        
        pauseButton.addActionListener(new ControlGame());
        playButton.addActionListener(new ControlGame());
        nextButton.addActionListener(new ControlGame());
        nextNButton.addActionListener(new ControlGame());

        pauseButton.setEnabled(false);
        playButton.setEnabled(false);
        nextButton.setEnabled(false);
        nextNButton.setEnabled(false);
        
        pane.add(pauseButton);
        pane.add(playButton);
        pane.add(nextButton);
        pane.add(nextNButton);
        pane.add(numSteps);
        
        JPanel temp = new JPanel();
        temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
        temp.add(Box.createRigidArea(new Dimension(0, 60)));
        pane.add(temp);
        
        return pane;
    }

    
    /**
     * Enable scroll bars base on map size
     * @param size the map size
     */
    public void scrollOn(int size) {
        if (size > 15) {
            hScroll.setEnabled(true);
            hScroll.setValues(0, 10, 0, (size * 32) - 460);
            hScroll.addAdjustmentListener(new ScrollBarChange());
            vScroll.setEnabled(true);
            vScroll.setValues(0, 10, 0, (size * 32) - 480);
            vScroll.addAdjustmentListener(new ScrollBarChange());
        }
    }
    /**
     * @author thaile
     *
     * This action listener are for commands dealing with species and map 
     * selection and processing
     */
    private class FileFinder implements ActionListener {
        /**
         * This tells us what to do
         * @param e the button that was clicked
         */
        public void actionPerformed(ActionEvent e) {
            String cmd = e.getActionCommand();
            JPanel temp = new JPanel();
            if (cmd == "MAP") {
                System.out.println("finding a map.zip");
             // JFileChooser mfinder = new JFileChooser(main.getMapPath());
                JFileChooser mfinder = new JFileChooser("bin/map/");
                int returnVal = mfinder.showOpenDialog(temp);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File file = mfinder.getSelectedFile();
                    String fileName = file.getName();
                    //This strips off the .zip extension
                    fileName = fileName.substring(0, fileName.length() - 4);
                    System.out.println(fileName);
                    DataUnzip.unzip(file, fileName);
                    System.out.println("done with zip");
                    MapMaker newMap = new MapMaker(fileName);
                    int err;
                    if ((err = newMap.readInMap()) != 1) {
                        System.out.println("invalid map file" + err);
                        return;
                    }
                    map = newMap.getMapInfo();
                    mapLabel.setText(map.getName());
                }
                
            } else if (cmd == "Species1") {
                System.out.println("finding a s1.zip");
                species1 = extractSpecies();
                if (species1 != null) {
                    species1Icon.setIcon(species1.getIcon());
                    species1Label.setText(species1.getSpeciesType());
                }
                
            } else if (cmd == "Species2") {
                System.out.println("finding a s2.zip");
                species2 = extractSpecies();
                if (species2 != null) {
                    species2Icon.setIcon(species2.getIcon());
                    species2Label.setText(species2.getSpeciesType());
                }
            }
        }
    }
    
    /**
     * Allows user to pick a zip file and extract and creates a 
     * DWSpecies object
     * @return DWSpecies or null
     */
    public DWSpecies extractSpecies() {
        JPanel temp = new JPanel();
        // TODO JFileChooser mfinder = new JFileChooser(main.getMapPath());
        JFileChooser sfinder = new JFileChooser("bin/creatures/");
        int returnVal = sfinder.showOpenDialog(temp);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = sfinder.getSelectedFile();
            String fileName = file.getName();
            //This strips off the .zip
            fileName = fileName.substring(0, fileName.length() - 4);
            System.out.println(fileName);
            DataUnzip.unzip(file, fileName);
            SpeciesMaker newSpecie = new SpeciesMaker(fileName);
            newSpecie.readInSpecies();
            return newSpecie.getSpecies();
        } 
        return null;
    }
    
    /**
     * Stops the game
     *
     */
    public void gameOver() {
        System.out.println("game over being called");
        if (endNow) {
            main.switchWindow(Conts.STARTUP_WIN);
        } else {
            theGame.setStop(true);
            main.switchWindow(Conts.LOCAL_WIN);
        }
        
        
        
    }
    
    /**
     * @author thaile
     *
     * Listener for the scroll bar so that the battle can be
     * notified
     */
    private class ScrollBarChange implements AdjustmentListener {
        /**
         * what does the processing
         * @param e the event
         */
        public void adjustmentValueChanged(AdjustmentEvent e) {
            if (e.getSource().equals(hScroll)) {
                mapDisplay.setXOff(hScroll.getValue());
            } else if (e.getSource().equals(vScroll)) {
                mapDisplay.setYOff(vScroll.getValue());
            }
        }
    }
    /**
     * @author thaile
     * 
     * this is for the number of creature selection
     */
    private class NumCreatureChanger implements ActionListener {
        /**
         * action to do
         * @param e the new value
         */
        public void actionPerformed(ActionEvent e) {
            int a = Integer.parseInt(e.getActionCommand());
            numCreatures = a;
             
        }
    }
    /**
     * @author thaile
     *
     * This action listener deals with creating and setting up a new game; 
     */
    private class CreateGame implements ActionListener {


        /**
         * This tells us what to do
         * @param e not used
         */
        public void actionPerformed(ActionEvent e) {
            if (map == null) {
                System.out.println("set map ");
            } else if (species1 == null) {
                System.out.println("species1");
            } else if (species2 == null) {
                System.out.println("species2");
            } else {
                System.out.println("create game");
                game = new Game("local");
                game.setSpecies1(species1);
                game.setSpecies2(species2);
                game.setMapInfo(map);
                game.setNumCreatures(numCreatures);
                game.addToParticipants(null);
                if (game.initGame() == 1) {

                    System.out.println("game set up");
                    mapDisplay.initBattle(game.getGameData());
                    createButton.setEnabled(false);

                    Thread drawThread = new Thread(mapDisplay);
                    drawThread.start();
                    playButton.setEnabled(true);
                    nextButton.setEnabled(true);
                    nextNButton.setEnabled(true);

                }

            }
            
            
        }
    }
    /**
     * @author thaile
     *
     * This action listener deals with controling the flow of the game; 
     */
    private class ControlGame implements ActionListener {
        /**
         * This tells us what to do
         * @param e not used
         */
        public void actionPerformed(ActionEvent e) {
            String cmd = e.getActionCommand();
            
            if (cmd.equals("pause")) {
                theGame.setStop(true);
                
            } else if (cmd.equals("play")) {
                System.out.println("play");
                pauseButton.setEnabled(true);
                playButton.setEnabled(false);
                nextButton.setEnabled(false);
                nextNButton.setEnabled(false);
                theGame = new PlayGame(mapDisplay, game, this);
                Thread runGame = new Thread(theGame);
                runGame.start();
                
            } else if (cmd.equals("next")) {
                String cmda = game.getACmd();
                System.out.println("local: " + cmda);
                if (cmda != null) {
                    mapDisplay.addToQueue(cmda);
                    mapDisplay.setServerWait(false);
                }
            } else if (cmd.equals("nextN")) {
                try {
                    int move = Integer.parseInt(numSteps.getText());
                    if (move < 1) {
                        move = 1;
                        numSteps.setText("1");
                    }
                    nextNButton.setEnabled(false);
                    nextButton.setEnabled(false);
                    playButton.setEnabled(false);
                    pauseButton.setEnabled(true);
                    theGame = new PlayGame(mapDisplay, game, this);
                    theGame.setMoves(move);
                    Thread runGame = new Thread(theGame);
                    runGame.start();
                    
                } catch (NumberFormatException e1) {
                    System.out.println("invalid number in box");
                }
            }      
        }
        
        /**
         * Inital state of the buttons
         * needed when coming back from wherever;
         */
        public void initButtonSetting() {
            if (pauseButton.isEnabled()) {
                pauseButton.setEnabled(false);
                playButton.setEnabled(true);
                nextButton.setEnabled(true);
                nextNButton.setEnabled(true);
            }
        }
    }
    /**
     * @author thaile
     *
     *Basically automaticaly clicks next for us
     */
    private class PlayGame implements Runnable {
        /** the battle display */
        private Battle theBattle;
        /** the game itself */
        private Game theGame;
        /** Control the game */
        private ControlGame control;
        /** wheather to stop or not */
        private boolean stop = false;
        /** or stop with counter */
        private int moves = 0;
        /**
         * creates a new play action
         * @param battle the battle display
         * @param game the game
         * @param control the control files
         */
        public PlayGame(Battle battle, Game game, ControlGame control) {
            theBattle = battle;
            theGame = game;
            this.control = control;
        }
        
        /**
         * gets the commands every two seconds which is how long
         * it take to animate one move
         */
        public void run() {
            while (!stop || moves > 0) {
                
                try {
                    String cmda = game.getACmd();
                    System.out.println(cmda);
                    if (cmda != null) {
                        mapDisplay.addToQueue(cmda);
                        mapDisplay.setServerWait(false);
                        moves--;
                    }
                    Thread.sleep(1250); //sleep for 2 sec
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            control.initButtonSetting();
            System.out.println("Stoping now");
        } 
        /**
         * Tells the thread to stop
         * @param value true or false
         */
        public void setStop(boolean value) {
            stop = value;
            moves = 0;
        }
        /**
         * this does the move x times
         * @param move the number of moves todo
         */
        public void setMoves(int move) {
            stop = true;
            moves = move;
        }
    }
    /**
     * @author thaile
     * Closes everything and go back to the main startup
     */
    private class EndGame implements ActionListener {
        /**
         * The method that shutdown everything
         * @param e the event
         */
        public void actionPerformed(ActionEvent e) {
            closeThis();
        }
    }
    
    /**
     * This closes all the running threads and exit the local battle
     *
     */
    public void closeThis() {
        System.out.println("close this ");
        if (game != null) {
            game.setPlaying(false);
        }
        if (theGame != null) {
            theGame.setStop(true);
        }
        if (mapDisplay.isPlaying()) {
            mapDisplay.emptyQueue();
            mapDisplay.addToQueue("0,winner,|none|");
            mapDisplay.setServerWait(false);
            endNow = true;
        } else {
            System.out.println("switch");
            main.switchWindow(Conts.STARTUP_WIN);
        }

    }
}
