package edu.cs2335.lab7.client;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

/**
 * This class asks the user for his/her username/password and then
 * sends them to the server for verification.
 * @author Michael Gorbsky - gtg950v
 * @version 2.4
 */
public class Login extends JFrame {
    /**This field is where the user enters their username.*/
    private JTextField usernameField;
    /**This field is where the user enters their password, displays stars.*/
    private JPasswordField passwordField;
    /**The username of the user.*/
    private String username;
    /**Provides a link to the ClientConnection class.*/
    private ClientConnection connect;
    /**The hight of the screen.*/
    private int screenheight;
    /**The width of the screen.*/
    private int screenwidth;
    /**The login frame */
    private Login frame = this;

    /**
     * Creates and sets up the JFrame for the user to input their
     * username, password, and port number.
     * @param connect ClientConnection -  a link to the class
     * @param screenheight int - the height of the screen
     * @param screenwidth int - the width of the screen
     */
    public Login(ClientConnection connect, int screenheight,
                 int screenwidth) {
        this.connect = connect;
        this.screenheight = screenheight;
        this.screenwidth = screenwidth;
        //sets up the JFrame and sets some default properties
        setTitle("Enter Username and Password");
        setResizable(false);
        setSize(new Dimension(500, 130));
        setLocation((screenwidth - 500) / 2, (screenheight - 130) / 2);
        setContentPane(createWindowContents());
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeWindow();
            }
         });
        setVisible(true);
    }
    /**
     * Creates the contens of the window and adds them to the JPanel.
     * @return JPanel - the panel with all the contents added to it
     */
    private JPanel createWindowContents() {
        JPanel usernamePassword = new JPanel();
        usernamePassword.setLayout(new GridLayout(4, 2, 0, 0));
        usernamePassword.add(new JLabel("Username:"));
        usernameField = new JTextField();
        usernamePassword.add(usernameField);
        usernamePassword.add(new JLabel("Password"));
        passwordField = new JPasswordField();
        usernamePassword.add(passwordField);
        //creates and adds all the JButtons
        JButton login = new JButton("Login");
        login.addActionListener(new LoginListener());
        usernamePassword.add(login);
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new CancelListener());
        usernamePassword.add(cancel);
        JButton register = new JButton("Register New User");
        register.addActionListener(new RegisterListener());
        usernamePassword.add(register);
        return usernamePassword;
    }
    /**
     * Returns the username of the user.
     * @return String - the username of the user.
     */
    public String getUsername() {
        return username;
    }
    /**
     * Disposes of the window when it is no longer needed.
     */
    public void closeWindow() {
        frame.setVisible(false);
        frame.dispose();
    }
    /**Calls the server for varification when the user presses register.*/
    private class LoginListener implements ActionListener {

        /**
          * When the user clicks the Login button, this ActionListener
          * sends the info to the server to login.
          * @param e ActionEvent - the source of the call (Login)
          */
        public void actionPerformed(ActionEvent e) {

            String passwordInput = "";
            String usernameInput = usernameField.getText();
            char[] password = passwordField.getPassword();
            for (int i = 0; i < password.length; i++) {
                passwordInput += password[i];
            }

            connect.setUsername(usernameInput);

            if (!usernameInput.equals("") && !passwordInput.equals("")) {
                connect.sendMessage("LOGIN~" + usernameInput + "~"
                                    + passwordInput);
            }
        }
    }

    /**Shuts down the UsernamePassword frame and opens the register frame.*/
    private class RegisterListener implements ActionListener {

        /**
          * This ActionEvent ends the program when the user clicks
          * the cancel button.
          * @param e ActionEvent - the source of the call (Register)
          */
        public void actionPerformed(ActionEvent e) {
            Register register = new Register(connect, screenheight,
                                             screenwidth);
            register.setVisible(true);
        }
    }

    /**Is called when the user hits the cancel button.*/
    private class CancelListener implements ActionListener {

        /**
          * This ActionEvent closes the window when the user clicks
          * the cancel button.
          * @param e ActionEvent - the source of the call (Cancel)
          */
        public void actionPerformed(ActionEvent e) {
            closeWindow();
        }
    }
}

