package edu.cs2335.lab7.client;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.UnknownHostException;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import edu.cs2335.lab7.creature.DWSpecies;

/**
 * @author macguy
 *
 * This is the main client driver class that control everything
 */
public class MainClass {
    /** DATA: host name to connect to */
    private String host = "localhost";
    /** DATA: the port number to use */
    private String portNum = "30303";
    /** DATA: map saved path */
    private String mapPath = "map/";
    /** DATA: creature saved path */
    private String crePath = "creature/";
    
    /** Connection to the network */
    private ClientConnection connection = null;
    /** The login window when the user wants to login */
    private Login loginwindow;
    /** The screenwidth */
    private int screenwidth;
    /** the screenheight */
    private int screenheight;
    /** The currently displaying window */
    private int currentWindow = 0;
    
    /** Reference to the startMenu gui */
    private StartMenu setup;
    /** Reference to the Lobby window gui */
    private LobbyWindow lobby;
    /** Reference to the running battle renderer */
    private Battle battle;
    /** Reference to the local battle window */
    private LocalBattle local;
    /** Reference to the Serverbattles */
    private ServerBattle serverbattle;
    
    /** Data: success? */
    private int success = 0;
    /** DATA: failure? */
    private int failure = 1;
    /** DATA: start a new connection */
    private boolean newconnection;
    
    /**
     * Starts the program
     * @param args command-line args
     */
    public static void main(String[] args) {
        MainClass mc = new MainClass();
        mc.loadSettings();
    }

    /** 
     * Startup the startup window and get the window sizes 
     */
    public MainClass() {
        
        screenwidth = Toolkit.getDefaultToolkit().getScreenSize().width;
        screenheight = Toolkit.getDefaultToolkit().getScreenSize().height;
        setup = new StartMenu(this, screenwidth, screenheight);
    }

    /** 
     * This displays the popup windows for user input 
     * @param window the window to display
     */
    public void loadWindow(int window) {
        switch(window) {
        
        case Conts.SETTING_WIN:
            SettingGui settings = new SettingGui(this, screenwidth, 
                    screenheight);
        settings.setVisible(true);
            break;
        case Conts.LOGIN_WIN:
            int result = openConnection();
            if (result == 0 && newconnection) {
            Thread connectthread = new Thread(connection);
            connectthread.start();
            newconnection = false;
            }
            if (result == success) {
                loginwindow = new Login(connection, screenheight, screenwidth);
            }
            break;
        default:
        }
    }
    
    /**
     * The switches out the main window to another
     * @param window The window to change to
     */
    public void switchWindow(int window) {
        
        switch (window) {
        case Conts.STARTUP_WIN:
            if (currentWindow == Conts.LOCAL_WIN) {
                local.dispose();
            }
               setup = new StartMenu(this, screenwidth, screenheight);
               break;
        case Conts.LOBBY_WIN:
               loginwindow.dispose();
               setup.dispose();
               lobby = new LobbyWindow(this, connection, screenwidth, 
                       screenheight);
               break;
        case Conts.LOCAL_WIN:
            if (currentWindow == Conts.STARTUP_WIN) {
               setup.dispose();
            } else if (local != null) {
                local.dispose();
                System.out.println("local dispose");
            }
            local = new LocalBattle(this, screenwidth, screenheight);
        default:
        }
        
        currentWindow = window;
    }
    /** 
     * Starts the connection to the server 
     * @return err code
     */
    public int openConnection() {
        try {
            if (connection == null) {
            connection = new ClientConnection(this, host, portNum);
            newconnection = true;
            return success;
            }
        } catch (NumberFormatException n) {
            displayError("SF", "numberformat");
            newconnection = false;
            connection = null;
            return failure;
        } catch (IllegalArgumentException i) {
            displayError("SF", "portrange");
            newconnection = false;
            connection = null;
            return failure;
        } catch (UnknownHostException h) {
            displayError("SF", "host");
            newconnection = false;
            connection = null;
            return failure;
        } catch (IOException h) {
            displayError("SF", "ioerror");
            newconnection = false;
            connection = null;
            return failure;
        }
        return success;
    }
    
    /**
     * Displays an error message
     * @param titl  the category of the error
     * @param errormessage  the errormessage to display
     */
    public void displayError(String titl, String errormessage) {
        String displaymessage = errormessage;
        String title = null;
        Component parent = lobby;
        if (titl == null) {
            title = "Error";
        } else if (titl.equals("SF")) {
            title = "Connection Failure";
        } else {
            title = titl;
        }
        if (titl.equals("CF")) {
            parent = lobby.getStable();
            parent.requestFocus();
        }
        if (errormessage.equals("numberformat")) {
            displaymessage = "You have entered an invalid port number.";
        } else if (errormessage.equals("portrange")) {
            displaymessage = "You must specify a port between 1 and 65535.";
        } else if (errormessage.equals("host")) {
            displaymessage = "No host found at that address.";
        } else if (errormessage.equals("ioerror")) {
            displaymessage = "Connection refused by host.";
        } else if (errormessage.equals("connecterror")) {
            displaymessage = "A connection error has occurred.";
        } else {
            displaymessage = errormessage;
        }
        if (JOptionPane.
                showConfirmDialog(parent,
                                  displaymessage,
                                  title,
                                  JOptionPane.DEFAULT_OPTION,
                                  JOptionPane.ERROR_MESSAGE)
                == JOptionPane.YES_OPTION) {
                return;
        }
    }
    
    
    /**
     * Displays a status message to the user (nonerror).
     * @param themessage the message recieved
     */
    public void displayMessage(String themessage) {
        if (JOptionPane.
            showConfirmDialog(lobby,
                              themessage,
                              "Notice",
                              JOptionPane.DEFAULT_OPTION,
                              JOptionPane.INFORMATION_MESSAGE)
            == JOptionPane.YES_OPTION) {
            return;
        }
    }
    
    /**
     * Setups the option window
     * @param optionslist the list of options
     */
    public void chooseOptions(String[] optionslist) {
        String maplist = "";
        String creaturelist = "";
        String numcreatures = "";
        String test = "";
        
        int runningindex = 2;
        while (!((test = optionslist[runningindex]).equals("ME"))) {
            maplist += test + "~";
            runningindex++;
        }
        runningindex++;
        while (!((test = optionslist[runningindex]).equals("CE"))) {
            creaturelist += test + "~";
            runningindex++;
        }
        runningindex++;
        for (int i = runningindex; i < optionslist.length; i++) {
            numcreatures += optionslist[i] + "~";
        }
        String[] mapdata = maplist.split("~");
        String[] creaturedata = creaturelist.split("~");
        String[] numcreaturedata = numcreatures.split("~");
        
        final JDialog optionsdialog = new JDialog(lobby, "Choose Game Options");
        optionsdialog.getContentPane()
           .setLayout(new BoxLayout(
                   optionsdialog.getContentPane(), BoxLayout.Y_AXIS));
        JLabel maplabel = new JLabel("Map:");
        JLabel creaturelabel = new JLabel("Creature:");
        JLabel numcreatlabel = new JLabel("# of Creatures:");
        final JComboBox choosemap = new JComboBox(mapdata);
        final JComboBox choosecreature = new JComboBox(creaturedata);
        final JComboBox choosenum = new JComboBox(numcreaturedata);
        JPanel mappanel = new JPanel();
        JPanel creaturepanel = new JPanel();
        JPanel numpanel = new JPanel();
        mappanel.setLayout(new BoxLayout(mappanel, BoxLayout.X_AXIS));
        creaturepanel.setLayout(new BoxLayout(creaturepanel, BoxLayout.X_AXIS));
        numpanel.setLayout(new BoxLayout(numpanel, BoxLayout.X_AXIS));
        mappanel.add(Box.createRigidArea(new Dimension(5, 0)));
        mappanel.add(maplabel);
        mappanel.add(Box.createRigidArea(new Dimension(10, 0)));
        mappanel.add(choosemap);
        creaturepanel.add(Box.createRigidArea(new Dimension(5, 0)));
        creaturepanel.add(creaturelabel);
        creaturepanel.add(Box.createRigidArea(new Dimension(10, 0)));
        creaturepanel.add(choosecreature);
        numpanel.add(Box.createRigidArea(new Dimension(5, 0)));
        numpanel.add(numcreatlabel);
        numpanel.add(Box.createRigidArea(new Dimension(10, 0)));
        numpanel.add(choosenum);
        JPanel ok = new JPanel();
        JButton okbutton = new JButton("OK");
        okbutton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        connection.sendMessage("GAMEOPTIONS~"
                                + choosenum.getSelectedItem() + "~"
                                + choosemap.getSelectedItem() + "~"
                                + choosecreature.getSelectedItem());
                        optionsdialog.dispose();
                    }
                });
        ok.add(okbutton);
        optionsdialog.getContentPane().add(mappanel);
        optionsdialog.getContentPane().add(creaturepanel);
        optionsdialog.getContentPane().add(numpanel);
        optionsdialog.getContentPane().add(ok);
        optionsdialog.setSize(new Dimension(250, 150));
        optionsdialog.setLocation((screenwidth - 250) / 2,
                (screenheight - 150) / 2);
        optionsdialog.setVisible(true);
    }
    
    /**
     * option list for choosing creatures
     * @param optionslist the creatures
     */
    public void chooseCreature(String[] optionslist) {
        String creaturelist = "";
        String test = "";
        
        int runningindex = 1;
        while (!((test = optionslist[runningindex]).equals("CE"))) {
            creaturelist += test + "~";
            runningindex++;
        }
        String[] creaturedata = creaturelist.split("~");
        
        final JDialog optionsdialog = new JDialog(lobby, 
                "Choose Game Creature");
        optionsdialog.getContentPane()
           .setLayout(new BoxLayout(
                   optionsdialog.getContentPane(), BoxLayout.Y_AXIS));
        JLabel creaturelabel = new JLabel("Creature:");
        final JComboBox choosecreature = new JComboBox(creaturedata);
        JPanel creaturepanel = new JPanel();
        creaturepanel.setLayout(new BoxLayout(creaturepanel, BoxLayout.X_AXIS));
        creaturepanel.add(Box.createRigidArea(new Dimension(5, 0)));
        creaturepanel.add(creaturelabel);
        creaturepanel.add(Box.createRigidArea(new Dimension(10, 0)));
        creaturepanel.add(choosecreature);
        JPanel ok = new JPanel();
        JButton okbutton = new JButton("OK");
        okbutton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        connection.sendMessage("SELCREAT~"
                                + choosecreature.getSelectedItem());
                        optionsdialog.dispose();
                    }
                });
        ok.add(okbutton);
        optionsdialog.getContentPane().add(creaturepanel);
        optionsdialog.getContentPane().add(ok);
        optionsdialog.setSize(new Dimension(250, 85));
        optionsdialog.setLocation((screenwidth - 250) / 2,
                (screenheight - 85) / 2);
        optionsdialog.setVisible(true);
    }
    
    /**
     * Returns the user name of the person selected
     * @return A user name
     */
    public String getSelectedUser() {
        return lobby.getUserChosen();
    }
    
    /**
     * creates a new online battle
     */
    public void createBattle() {
        serverbattle = new ServerBattle(this, screenwidth, screenheight);
    }
   
    /**
     * Displays a chat message to the lobby chat window.
     * @param txt  the message
     */
    public void displayMainChatMessage(String[] txt) {
        lobby.addText(txt);
    }
    
    /**
     * Displays a chat message to the game chat window.
     * @param txt  the message
     */
    public void displayGameChatMessage(String[] txt) {
        // GAMEMSG~username~msg
        serverbattle.addText(txt);
    }
    
    
    /**
     * Updates the list of online users in the lobby chatroom.
     * @param txt  the list of users
     */
    public void updateActiveUsers(String[] txt) {
        final String[] userlist = txt;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (lobby != null) {
                lobby.updateUsers(userlist);
                }
            }
        }); 
    }
    
    /**
     * Updates the active game window
     * @param txt the name of the active game
     */
    public void updateActiveGames(String[] txt) {
        final String[] userlist = txt;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (lobby != null) {
                lobby.updateGames(userlist);
                }
            }
        }); 
    }
    
    /**
     * Gets the selected species
     * @return a DWSpecies
     */
    public DWSpecies getNewCreature() {
        return lobby.getStable().getAddedCreature();
    }
    
    /**
     * This closes the windows
     */
    public void closeWindows() {
        if (setup != null) {
            setup.dispose();
        }
        if (lobby != null) {
            lobby.closeOnQuit();
        }
        if (local != null) {
            local.dispose();
        }
        if (serverbattle != null) {
            serverbattle.dispose();
        }
        saveSettings();
        System.exit(0);
    }
    
    /**
     * Inital the stables
     */
    public void setStableNull() {
        if (lobby != null) {
            lobby.closeStable();
        }
    }
    
    /**
     * Updates the list of online users in the game chatroom.
     * @param txt  the list of users
     */
    public void updateGameUsers(String[] txt) {
        if (serverbattle != null) {
        serverbattle.updateUsers(txt);
        }
        //code here to update the list of users in the game chatroom
    }
    
    /**
     * Send a msg to the server
     * @param msg the messages
     */
    public void sendToServer(String msg) {
        connection.sendMessage(msg);
    }
    /**
     * Closes the login
     */
    public void closeLoginWindow() {
        loginwindow.dispose();
    }
    
    /**
     * Create a new battle object
     */
    public void startNewBattle() {
        battle = new Battle(null);
    }
    
    /**
     * This adds commands for the battle to 
     * the battles queue
     * @param cmds The cmds to add
     */
    public void battleCmds(String[] cmds) {
        int index = 1;
        while (!cmds[index].equals("ME")) {
            serverbattle.addToQueue(cmds[index]);
            index += 1;
        }
        serverbattle.ready();
    }
    
    /**
     * This updates the creature list 
     * @param creatures The user creatures
     */
    public void displayCreatureList(String[] creatures) {
        lobby.getStable().displayCreatures(creatures);
    }
    
    /**
     * loads the user network setting from file 
     */
    public final void loadSettings() {
        try {
            File settingsfile = new File("clientSettings.set");
            FileReader fileReader = new FileReader(settingsfile);
            BufferedReader inFile = new BufferedReader(fileReader);

            host = inFile.readLine();
            portNum = inFile.readLine();
            mapPath = inFile.readLine();
            crePath = inFile.readLine();

            System.out.println(host + " " + portNum + " " + mapPath + " " 
                    + crePath);
            inFile.close();
        } catch (FileNotFoundException e) {
            System.out.println("file not found create new one");

        } catch (IOException e) {
            System.out.println("error reading file");
        }
    }
    
    /**
     * Save the settings to file
     */
    public final void saveSettings() {
        try {
            File settingsfile = new File("clientSettings.set");
            FileWriter fileWriter = new FileWriter(settingsfile);
            PrintWriter outFile = new PrintWriter(fileWriter);
            outFile.write(host + "\n");
            
            outFile.write(portNum + "\n");
            outFile.write(mapPath + "\n");
            outFile.write(crePath + "\n");
            outFile.flush();
            outFile.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            System.exit(0);
        }
    }
    /**
     * @return Returns the lobby.
     */
    public LobbyWindow getLobby() {
        return lobby;
    }
    /**
     * @return Returns the connection.
     */
    public ClientConnection getConnection() {
        return connection;
    }
    
    /**
     * @return Returns the crePath.
     */
    public String getCrePath() {
        return crePath;
    }
    /**
     * @param crePath The crePath to set.
     */
    public void setCrePath(String crePath) {
        this.crePath = crePath;
    }
    /**
     * @return Returns the host.
     */
    public String getHost() {
        return host;
    }
    /**
     * @param host The host to set.
     */
    public void setHost(String host) {
        this.host = host;
    }
    /**
     * @return Returns the mapPath.
     */
    public String getMapPath() {
        return mapPath;
    }
    /**
     * @param mapPath The mapPath to set.
     */
    public void setMapPath(String mapPath) {
        this.mapPath = mapPath;
    }
    /**
     * @return Returns the portNum.
     */
    public String getPortNum() {
        return portNum;
    }
    /**
     * @param portNum The portNum to set.
     */
    public void setPortNum(String portNum) {
        this.portNum = portNum;
    }

    /**
     * @return Returns the battle.
     */
    public ServerBattle getBattle() {
        return serverbattle;
    }
}