package edu.cs2335.lab7.client;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferStrategy;
import java.io.Serializable;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.cs2335.lab7.data.MapData;
import edu.cs2335.lab7.data.MapImage;
import edu.cs2335.lab7.data.MapMaker;

/**
 * @author thaile
 * 
 * This loads and draws the map to a given graphic component
 */
public class MapRender implements Serializable {

    /** The map info */
    private MapImage map;
    /** Holds the mapdata */
    private int[][] mapdata;
    /** Holds the tiles to use */
    private Image[] tiles;
    /** Holds the size of the map */
    private int mapSize;
    
    
    /**
     * Demos the map rendering 
     * @param args command-line args
     */
    public static void main(String[] args) {
        BufferStrategy strategy;
        JFrame frame = new JFrame("Map Renderer Demo");
        JPanel pane = (JPanel) frame.getContentPane();
        
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        String id = "test";  //make sure the map data is in the test folder 
                             // before running
        MapMaker mapReader = new MapMaker(id);
        Canvas test = new Canvas();
        int err = mapReader.readInMap();
        if (err != 1) {
            System.out.println("error reading map " + err);
            System.exit(0);
        }
         
        MapData mapData = mapReader.getMapInfo();
        
        MapRender map = new MapRender(mapData.getMapImage());
        test.setBounds(0, 0, 800, 600);
        pane.add(test, BorderLayout.CENTER);
        pane.add(new JLabel("test"), BorderLayout.WEST);
        frame.pack();
        frame.setVisible(true);
        //frame.setResizable(false);

        test.setIgnoreRepaint(true);
        test.createBufferStrategy(2);
        strategy = test.getBufferStrategy();
        Graphics2D g2d = (Graphics2D) strategy.getDrawGraphics();
        

        while (true) {
            g2d.setColor(Color.black);
            g2d.fillRect(0, 0, 800, 600);
            map.drawMap(g2d, 0, 0);
            
            g2d.dispose();
            strategy.show();
            
            try { 
                Thread.sleep(100); 
            } catch (Exception e) {
                System.out.println("couldn't sleep");
            }
        }
    }
    
    
    /**
     * Creates the new map class and
     * 
     * @param map The name of the battle
     */
    public MapRender(MapImage map) {
        this.map = map;
        this.mapdata = map.getMap();
        this.mapSize = mapdata.length; //Indirectly get the map size
        ImageIcon[] tileA = map.getTiles();
        tiles = new Image[tileA.length];
        for (int i = 0; i < tileA.length; i++) {
            tiles[i] = tileA[i].getImage();
        }
            
    }

    /**
     * Called by the game class to draw a map to the given screen
     * 
     * @param display Draw the map here
     * @param xOff the offset on the x-axis
     * @param yOff the offset on the y-axis
     */
    public void drawMap(final Graphics2D display, int xOff, int  yOff) {
        for (int i = 0; i < mapSize; i++) {
            for (int j = 0; j < mapSize; j++) {
                display.drawImage(tiles[mapdata[j][i]], j * 32 - xOff,
                        i * 32 - yOff, null);
            }
        }
        
    }

}

