package edu.cs2335.lab7.client;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;


/**
 * This class asks the user for his/her username/password and then
 * sends them to the server for verification.
 * @author Michael Gorbsky - gtg950v
 * @author Jonathan Reitnauer - gtg739g
 * @version 3.0
 */
public class Register extends JFrame {

    /**This field is where the user enters their username.*/
    private JTextField usernameField;

    /**This field is where the user enters their password, displays stars.*/
    private JPasswordField passwordFieldA;

    /**This field is where the user re-enters their password, displays stars.*/
    private JPasswordField passwordFieldB;

    /**The username of the user.*/
    private String username;
    /**Provides a link to the ClientConnection class.*/
    private ClientConnection connect;
    /**Needed to reference the window inside the ActionEvents.*/
    private JFrame window;
    

    /**
     * Creates and sets up the JFrame for the user to input their
     * username, password, and port number.
     * @param connect ClientConnection -  a link to the class
     * @param screenheight int - the height of the screen
     * @param screenwidth int - the width of the screen
     */
    public Register(ClientConnection connect, int screenheight,
                    int screenwidth) {
        this.connect = connect;
        window = this;
        //sets up the JFrame and sets some default properties
        setTitle("Please Register for a new login.");
        setContentPane(createWindowContents());
        setResizable(false);
        setSize(new Dimension(500, 130));
        setLocation((screenwidth - 500) / 2, (screenheight - 130) / 2);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                cleanUp();
            }
         });
    }
    /**
     * Creates the contens of the window and adds them to the JPanel.
     * @return JPanel - the panel with all the contents added to it
     */
    private JPanel createWindowContents() {

        JPanel usernamePassword = new JPanel();
        usernamePassword.setLayout(new GridLayout(4, 2, 0, 0));

        //creates and adds all the labels and text fields
        usernamePassword.add(new JLabel("Username:"));
        usernameField = new JTextField();
        usernamePassword.add(usernameField);
        usernamePassword.add(new JLabel("Password"));
        passwordFieldA = new JPasswordField();
        usernamePassword.add(passwordFieldA);
        usernamePassword.add(new JLabel("Re-enter Password:"));
        passwordFieldB = new JPasswordField();
        usernamePassword.add(passwordFieldB);
        //creates and adds all the JButtons
        JButton login = new JButton("Register");
        login.addActionListener(new RegisterListener());
        usernamePassword.add(login);
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new CancelListener());
        usernamePassword.add(cancel);
        return usernamePassword;
    }

    /**
     * Returns the text typed into the username text box.
     * @return String - the text typed into the username text box.
     */
    public String getUsernameInput() {
        return usernameField.getText();
    }

    /**
     * Deconstructs the UsernamePassword window.
     */
    private void cleanUp() {
          window.setVisible(false);
          window.dispose();
    }

    /**Calls the server for varification when the user presses Register.*/
    private class RegisterListener implements ActionListener {

        /**
          * When the user clicks the Login button, this ActionListener
          * sends the info to the server to login.
          * @param e ActionEvent - the source of the call (Register)
          */
        public void actionPerformed(ActionEvent e) {

            String usernameInput  = getUsernameInput();
            char[] password1 = passwordFieldA.getPassword();
            String passwordInputA = "";
            char[] password2 = passwordFieldB.getPassword();
            String passwordInputB = "";
            for (int i = 0; i < password1.length; i++) {
                passwordInputA += password1[i];
            }
            for (int i = 0; i < password2.length; i++) {
                passwordInputB += password2[i];
            }

            if (usernameInput.equals("") || passwordInputA.equals("")
                    || passwordInputB.equals("")) {
                JOptionPane.showMessageDialog(
                    window, "All fields are required.",
                    "Empty Fields", JOptionPane.INFORMATION_MESSAGE);
            } else if (!passwordInputA.equals(passwordInputB)) {
                JOptionPane.showMessageDialog(
                    window, "Please re-enter your password.",
                    "Passwords Do Not Match!",
                    JOptionPane.INFORMATION_MESSAGE);
                passwordFieldA.setText("");
                passwordFieldB.setText("");
            } else {
                username = usernameInput;
                connect.sendMessage("NEWUSER~" + username + "~"
                                    + passwordInputA);
                cleanUp();
            }
        }
    }

    /**Is called when the user hits the cancel button.*/
    private class CancelListener implements ActionListener {

        /**
          * This ActionEvent closes the window when the user clicks
          * the cancel button.
          * @param e ActionEvent - the source of the call (Cancel)
          */
        public void actionPerformed(ActionEvent e) {
            cleanUp();
        }
    }
}
