/*
 * Created on Mar 9, 2005
 *
 * The Server Battle GUI.
 */
package edu.cs2335.lab7.client;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import edu.cs2335.lab7.server.GameData;

/**
 * @author thaile
 * Creates a new game locally by pretending to be the server.  
 */
public class ServerBattle extends JFrame {
    /** The Canvas for drawing */
    private Battle mapDisplay;
    /** The link to MainClass */
    private MainClass main;
    
    /** Horizontal scroll bar */
    private JScrollBar hScroll;
    /** Vertical scroll bar */
    private JScrollBar vScroll;

    /**Main Chat window.*/
    private JTextArea chatpane;
    /**The button that sends what is in the inputText box.*/
    private JButton send;
    /**The box where the user types in his/her chat.*/
    private JTextArea inputText;
    /**The area that displays the connected users.*/
    private JPanel userpane;
    /**This is primary user display part of the window.*/
    private JScrollPane userdisplay;
    /**A list of connected users..*/
    private JList users;
    
    /**
     * Test the gui
     * @param args command-line args
     */
    public static void main(String[] args) {
        new ServerBattle(null, 1000, 1000);
    }
    
    /**
     * Starts up a local battle gui and controls the game
     * @param main the link to the mainClass
     * @param screenwidth the screenwidth
     * @param screenheight the screenheight
     */
    public ServerBattle(MainClass main, int screenwidth, int screenheight) {
        super("Server Match");
        this.main = main;
        JPanel mainPane = new JPanel();
        mainPane.setPreferredSize(new Dimension(800, 680));
        mainPane.setLayout(new BorderLayout());
        final MainClass mw = main;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                mapDisplay.emptyQueue();
                mapDisplay.addToQueue(0 + ",winner,|none|");
                mapDisplay.setServerWait(false);
                mw.sendToServer("LEAVEGAME");
             }
        });
        mainPane.add(gameChatPane(), BorderLayout.SOUTH);
        mainPane.add(topDisplay(), BorderLayout.CENTER);
        getContentPane().add(mainPane);
        setSize(new Dimension(800, 600));
        setResizable(false);
        setLocation((screenwidth - 800) / 2, (screenheight - 680) / 2);
        pack();
        show();
        
    }
    
    /**
     * Stops the game
     *
     */
    public void gameOver() {
        System.out.println("game over being called");
        dispose();
        
    }
    
    /** Creates the chat pane 
     * @return A JPanel
     */
    private final JPanel gameChatPane() {
        JPanel chatpanel = new JPanel();
        chatpanel.setPreferredSize(new Dimension(800, 170));
        chatpanel.setLayout(new BoxLayout(chatpanel, BoxLayout.X_AXIS));
        chatpanel.add(gameChatDisplay());
        chatpanel.add(gameUsers());
        return chatpanel;
    }
    
    /**
     * The map display
     * @return A JPanel
     */
    private final JPanel topDisplay() {
        JPanel topdisplay = new JPanel();
        topdisplay.setPreferredSize(new Dimension(800, 500));
        JPanel gamedisplay = mapPane();
        JPanel stats = new JPanel();
        stats.setPreferredSize(new Dimension(300, 500));
        topdisplay.add(gamedisplay);
      //  topdisplay.add(stats);
        return topdisplay;
    }
    
    /** Creates the chat display 
     * @return A JPanel
     */
    private final JPanel gameChatDisplay() {
            JPanel westSide = new JPanel();
            
            JPanel southwest = new JPanel();
            southwest.setPreferredSize(new Dimension(580, 55));
            southwest.setLayout(new BoxLayout(southwest, BoxLayout.X_AXIS));
            southwest.setBorder(new EmptyBorder(6, 5, 6, 6));
            inputText = new JTextArea();
            inputText.setLineWrap(true);
            inputText.setPreferredSize(new Dimension(450, 50));
            inputText.setBorder(BorderFactory.createLineBorder(Color.black));
            southwest.add(inputText);
            send = new JButton("Send");
            send.setPreferredSize(new Dimension(90, 30));
            send.addActionListener(new SendListener());
            southwest.add(Box.createRigidArea(new Dimension(10, 2)));
            southwest.add(send);
            southwest.add(Box.createRigidArea(new Dimension(5, 2)));

            JPanel northwest = new JPanel();
            chatpane = new JTextArea();
            chatpane.setEditable(false);
            chatpane.setLineWrap(true);
            JScrollPane chatdisplay = new JScrollPane(chatpane);
            chatdisplay.setPreferredSize(new Dimension(600, 100));
            northwest.add(chatdisplay);
            
            westSide.setLayout(new BorderLayout());
            westSide.add(southwest, BorderLayout.SOUTH);
            westSide.add(northwest, BorderLayout.CENTER);
            return westSide;
        }
    
    /**
     * The list of game Users
     * @return A JPanel
     */
    private JPanel gameUsers() {
        JPanel east = new JPanel();
        east.setPreferredSize(new Dimension(200, 100));
        east.setLayout(new BoxLayout(east, BoxLayout.Y_AXIS));
        userpane = new JPanel();
        users = new JList();
        userdisplay = new JScrollPane(users);
        userdisplay.setPreferredSize(new Dimension(150, 200));
        userdisplay.setBorder(new TitledBorder("Connected Players:"));
        userpane.add(userdisplay);
        east.add(userpane);
        return east;
    }
    
    /**
     * Update the user list 
     * @param userlist the new user list
     */
    public final void updateUsers(String[] userlist) {
        String[] listdata = null;
        if (userlist.length < 3) {
            listdata = new String[1];
            listdata[0] = "";
        } else {
            listdata = new String[userlist.length - 2];
            int newindex = 0;
            for (int i = 1; i < userlist.length - 1; i++) {
                listdata[newindex] = userlist[i];
                newindex++;
            }
        }
           users.setListData(listdata);
           users.repaint();
    }
    
    /**
     * Add commands to battle queue
     * @param command the cmd to add
     */
    public void addToQueue(String command) {
        mapDisplay.addToQueue(command);
    }
    
    /**
     * Sets the server to be ready.
     */
    public void ready() {
        mapDisplay.setServerWait(false);
    }
    /**
     * Add text to the text field
     * @param input the text to add
     */
    public void addText(String[] input) {
        if (!chatpane.equals("")) {
            chatpane.append("\n");
        }
        chatpane.append(input[1] + ": " + input[2]);
        chatpane.setCaretPosition((chatpane.getDocument()).getLength() - 1);
}
    
    /**
     * get the chat text
     * @return a string
     */
    public String getChatText() {
        String temp = inputText.getText();
        inputText.setText("");
        return temp;
    }
    
    /**
     * This creates the panel for displaying the window
     * @return A JPanel
     */
    public final JPanel mapPane() {
        JPanel pane = new JPanel();
        mapDisplay = new Battle(this);
        mapDisplay.setBounds(0, 0, 480, 480);
        
        JPanel offSethScroll = new JPanel();
        offSethScroll.setLayout(new BoxLayout(offSethScroll, BoxLayout.X_AXIS));
        hScroll = new JScrollBar(JScrollBar.HORIZONTAL);
        hScroll.setPreferredSize(new Dimension(480, 20));
        hScroll.setEnabled(false);
        offSethScroll.add(hScroll);
        offSethScroll.add(Box.createRigidArea(new Dimension(20, 0)));

        vScroll = new JScrollBar(JScrollBar.VERTICAL);
        vScroll.setPreferredSize(new Dimension(20, 480));
        vScroll.setEnabled(false);
    
        TitledBorder tb = BorderFactory.createTitledBorder("map");
        pane.setBorder(tb);
        pane.setLayout(new BorderLayout());
        pane.setPreferredSize(new Dimension(500, 500));
        pane.add(mapDisplay, BorderLayout.CENTER);
        pane.add(offSethScroll, BorderLayout.SOUTH);
        pane.add(vScroll, BorderLayout.EAST);
        
        return pane;
    }
       
    /**
     * Enable scroll bars base on map size
     * @param size the map size
     */
    public void scrollOn(int size) {
        if (size > 15) {
            hScroll.setEnabled(true);
            hScroll.setValues(0, 10, 0, (size * 32) - 460);
            hScroll.addAdjustmentListener(new ScrollBarChange());
            vScroll.setEnabled(true);
            vScroll.setValues(0, 10, 0, (size * 32) - 480);
            vScroll.addAdjustmentListener(new ScrollBarChange());
        }
    }
  
    /**
     * Starts the battle display
     * @param data the data to use
     */
    public void initBattle(GameData data) {
        mapDisplay.initBattle(data);
        Thread display = new Thread(mapDisplay);
        display.start();
    }
    
    /**
     * 
     */
    public void sendReady() {
        main.sendToServer("READY");
    }
    
    /**This is the listener for the send button */
    private class SendListener implements ActionListener {
        
        /**
          * This ActionEvent sends the text to the server.
          * @param e ActionEvent - the source of the call (Send)
          */
         public void actionPerformed(ActionEvent e) {
             String message = getChatText();
             if (message != null && message != "") {
                (main.getConnection()).sendMessage("GAMEMSG~" + message);
             }
         }
     }
    
    /**
     * @author thaile
     *
     * Listener for the scroll bar so that the battle can be
     * notified
     */
    private class ScrollBarChange implements AdjustmentListener {
        /**
         * what does the processing
         * @param e the event
         */
        public void adjustmentValueChanged(AdjustmentEvent e) {
            if (e.getSource().equals(hScroll)) {
                mapDisplay.setXOff(hScroll.getValue());
            } else if (e.getSource().equals(vScroll)) {
                mapDisplay.setYOff(vScroll.getValue());
            }
        }
    }
    /**
     * @author thaile
     * Closes everything and go back to the main startup
     */
    private class EndGame implements ActionListener {
        /**
         * The method that shutdown everything
         * @param e the event
         */
        public void actionPerformed(ActionEvent e) {
            
        }
    }
}
