package edu.cs2335.lab7.client;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * This Class creates the Setting gui then accepts user input Handles new user
 * registration and network settings.
 * 
 * @author thaile
 */
public class SettingGui extends JFrame implements ActionListener {
    /** The client to send the data to */
    private final MainClass client;

    /** setup the server name */
    private final JTextField serverField;

    /** setup the port number */
    private final JTextField portField;

    /** where are all the map stored */
    private final JTextField mapField;

    /** where are all the creatures stored */
    private final JTextField creField;

    /** The default setting button */
    private final JButton defaultButt;

    /** The accept changes button */
    private final JButton acceptButton;
    
    /** This window. */
    private SettingGui frame;

    /**
     * This sets up and creates the Setting Gui
     * @param theClient Where to send info to
     * @param screenwidth The width of the screen
     * @param screenheight The height of the screen
     */
    public SettingGui(MainClass theClient, int screenwidth, int screenheight) {
        super("Settings Window");
        setResizable(false);
        frame = this;
        client = theClient;
        JPanel mainPane = new JPanel();
        mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.Y_AXIS));

        if (client == null) {

            serverField = new JTextField();
            portField = new JTextField();
            mapField = new JTextField();
            creField = new JTextField();
        } else { // demoing the settingGUI

            serverField = new JTextField(client.getHost());
            portField = new JTextField("" + client.getPortNum());
            mapField = new JTextField(client.getMapPath());
            creField = new JTextField(client.getCrePath());
        }

        JPanel top = new JPanel();
        top.setLayout(new GridLayout(2, 2, 5, 2));
        top.add(new JLabel("server:", JLabel.RIGHT));
        top.add(serverField);
        top.add(new JLabel("port:", JLabel.RIGHT));
        top.add(portField);

        JPanel mid = new JPanel();
        mid.setLayout(new BoxLayout(mid, BoxLayout.PAGE_AXIS));
        mid.add(new JLabel("Path to maps files"));
        mid.add(mapField);
        mid.add(new JLabel("Path to creature files"));
        mid.add(creField);

        defaultButt = new JButton("Default");
        acceptButton = new JButton("Accept");

        defaultButt.addActionListener(this);
        defaultButt.setActionCommand("default");
        acceptButton.addActionListener(this);
        acceptButton.setActionCommand("accept");

        JPanel bottom = new JPanel();
        bottom.setLayout(new BoxLayout(bottom, BoxLayout.LINE_AXIS));
        bottom.add(defaultButt);
        bottom.add(acceptButton);

        mainPane.add(top);
        mainPane.add(mid);
        mainPane.add(bottom);

        getContentPane().add(mainPane);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                frame.dispose();
            }
        });
        pack();
        setLocation((screenwidth - 200) / 2, (screenheight - 100) / 2);
    }

    /**
     * Process the data depending on which button was clicked
     * 
     * @param event
     *            The event that cause the action
     */
    public void actionPerformed(final ActionEvent event) {
        final String action = event.getActionCommand();

        if (action.equals("default")) {
            serverField.setText("localhost");
            portField.setText("30303");
            mapField.setText("map/");
            creField.setText("creature/");
        } else if (action.equals("accept")) {
            if (client != null) { //demoing the settings
                //need to validate the information
                client.setHost(serverField.getText());
                client.setPortNum(portField.getText());
                client.setMapPath(mapField.getText());
                client.setCrePath(creField.getText());
                frame.dispose();
            }
        }
    }

}