package edu.cs2335.lab7.client;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
//import java.net.URL;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import edu.cs2335.lab7.creature.DWSpecies;
import edu.cs2335.lab7.data.SpeciesMaker;

/**
 * This is the client's stable where they can add creatures,
 * list the server's creatures, and deleting creatures.
 * @author notme
 */
public class Stable extends JFrame {
    /**The main class that all GUI's access*/
    private MainClass main;
    /**The stable window */
    private final Stable window = this;
    /**The species list panel */
    private JPanel specieslist;
    /**The species */
    private JList species;
    /**The selected species */
    private DWSpecies selectedspecies;

    /**
     * Constructor
     * @param main the main class
     * @param screenwidth the screen width
     * @param screenheight the screeen height
     */
    public Stable(MainClass main, int screenwidth, int screenheight) {
        super();
        this.main = main;
        JPanel addpanel = new JPanel();
        JPanel removepanel = new JPanel();
        JPanel options = new JPanel();
        addpanel.setLayout(new BoxLayout(addpanel, BoxLayout.X_AXIS));
        removepanel.setLayout(new BoxLayout(removepanel, BoxLayout.X_AXIS));
        options.setLayout(new BoxLayout(options, BoxLayout.Y_AXIS));
        JButton addbutton = new JButton("Add Species");
        addbutton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        selectedspecies = extractSpecies();
                        if (selectedspecies != null) {
                        testName();
                        }
                    } });
        JButton removebutton = new JButton("Remove Species");
        removebutton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        String selectedcreat = (String) 
                            (species.getSelectedValue());
                        selectedcreat = selectedcreat.substring(0,  
                                        selectedcreat.indexOf(' '));
                        if (selectedcreat != null) {
                        deleteCreature(selectedcreat);
                        }
                    } });
        addpanel.add(addbutton);
        addpanel.add(Box.createRigidArea(new Dimension(35, 2)));
        removepanel.add(removebutton);
        removepanel.add(Box.createRigidArea(new Dimension(35, 2)));
        options.add(Box.createRigidArea(new Dimension(0, 65)));
        options.add(addpanel);
        options.add(Box.createRigidArea(new Dimension(0, 50)));
        options.add(removepanel);
        specieslist = new JPanel();
        specieslist.setLayout(new BoxLayout(specieslist, BoxLayout.Y_AXIS));
        specieslist.setPreferredSize(new Dimension(250, 350));
        species = new JList();
        JScrollPane list = new JScrollPane(species);
        specieslist.add(list);
        setTitle("Stable Manager");
        this.setResizable(false);
        this.getContentPane().setLayout(new BorderLayout());
        final MainClass mw = main;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                mw.setStableNull();
                window.dispose();
            }
        });
        setSize(450, 400);
        getContentPane().add(options, BorderLayout.EAST);
        getContentPane().add(specieslist, BorderLayout.WEST);
        setLocation((screenwidth - 500) / 2, (screenheight - 400) / 2);
        setVisible(true);
    }
    
    /**
     * Unzips and creates a DWSpecies object
     * @return A DWSPecies
     */
    public DWSpecies extractSpecies() {
        JPanel temp = new JPanel();
        JFileChooser sfinder = new JFileChooser("bin/creatures/");
        int returnVal = sfinder.showOpenDialog(temp);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = sfinder.getSelectedFile();
            String fileName = file.getName();
            //This strips off the .zip
            fileName = fileName.substring(0, fileName.length() - 4);
            System.out.println(fileName);
            DataUnzip.unzip(file, fileName);
            SpeciesMaker newSpecie = new SpeciesMaker(fileName);
            newSpecie.readInSpecies();
            return newSpecie.getSpecies();
        } 
        return null;
    }
    
    /**
     * Display a list of creatures in you stable
     * @param creatures the creatures 
     */
    public void displayCreatures(String[] creatures) {
        String[] listdata = new String[creatures.length - 2];
        int newindex = 0;
        for (int i = 1; i < creatures.length - 1; i++) {
            listdata[newindex] = creatures[i];
            newindex++;
        }
        species.setListData(listdata);
        species.repaint();
        requestFocus();
    }
    
    /**
     * See if the creature exist already
     */
    private void testName() {
        main.sendToServer("ADDCREAT~" + selectedspecies.getSpeciesType());
    }
    
    /**
     * Remove a creature from the server stable
     * @param selectedcreature the creature to remove
     */
    private void deleteCreature(String selectedcreature) {
        main.sendToServer("DELCREAT~" + selectedcreature);
    }
    
    /**
     * The DWCreature to add
     * @return A DWSpecies
     */
    public DWSpecies getAddedCreature() {
        return selectedspecies;
    }
  }
