package edu.cs2335.lab7.client;

import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * @author thaile
 * 
 * The Start Menu GUI.
 */
public class StartMenu extends JFrame {
    /** Image holding the world */
    private ImageIcon startScreen;
    /** Reference to the control class */
    private MainClass main;

    /**
     * Testing gui
     * @param args command-line args
     */
    public static void main(String[] args) {
        new StartMenu(null, Toolkit.getDefaultToolkit().getScreenSize()
                .width, Toolkit.getDefaultToolkit().getScreenSize().height);
    }
    /**
     * Creates a new window for the Darwin world game
     * @param main The control class
     * @param screenwidth The width of the whole screen
     * @param screenheight The height of the whole screen
     */
    public StartMenu(MainClass main, int screenwidth, int screenheight) {
        super("DARWIN WORLD");
        this.main = main;
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setLocation((screenwidth - 800) / 2, (screenheight - 600) / 2);
        loadImages();
 
        JPanel anew = new JPanel();
        JLabel image = new JLabel(startScreen);
        anew.add(image);
        image.requestFocusInWindow();
        anew.addMouseListener(new MouseListen());
        getContentPane().add(anew);
        pack();
        setResizable(false);
        setVisible(true);

    }

    
    /**
     * Loads the startScreen.GIF image
     */
    public final void loadImages() {
            URL url = this.getClass().getClassLoader().getResource(
                    "images/startScreen.gif");
            if (url == null) {
                System.out.println("bad url");
            }
            startScreen = new ImageIcon(url);
    }
    
    /**
     * This is the mouse listener class
     * 
     */
    private class MouseListen extends MouseAdapter {
        
        /**
         * For when a mouse is clicked on an component
         * @param e The mouseEvent
         */
        public void mouseClicked(MouseEvent e) {
            final int x = e.getX();
            final int y = e.getY();
            
            if (x > 510 && x < 720) {
                if (y > 190 && y < 225) {
                    main.loadWindow(Conts.LOGIN_WIN);
                    return;
                } else if (y > 250 && y < 275) {
                    main.switchWindow(Conts.LOCAL_WIN);
                    return;
                } else if (y > 365 && y < 390) {
                    main.loadWindow(Conts.SETTING_WIN);
                } else if (y > 420 && y < 445) {
                    main.closeWindows();
                }
            }
        }

    }
}