package edu.cs2335.lab7.creature;

import java.io.Serializable;

/**
 * Represents a edu.cs2335.lab7.creature's move
 *
 * @author Brandyn Roemer Date: Mar 5, 2005 Time: 7:28:52 PM
 */
public class CreatureMoveEvent implements Serializable {

    /** Constant indicating a edu.cs2335.lab7.creature moved */
    public static final int CREATURE_HOP = 1;

    /** Constant indicating a creature infected another creature */
    public static final int CREATURE_INFECT = 2;

    /** Constant indicating a creature turned/changed it's direction */
    public static final int CREATURE_TURN = 3;

    /** Constant indicating a creature was infected by another creature */
    public static final int CREATURE_INFECTED_BY = 4;


    /** Constant indicating that the creature turned left */
    public static final int TURN_LEFT = 1;

    /** Constant indicating that the creature turned right */
    public static final int TURN_RIGHT = 2;



    /** The type of event (move/infect/etc) */
    private int eventType;

    /** The edu.cs2335.lab7.creature the made the move */
    private DWCreature creature;

    /** The target - either a edu.cs2335.lab7.creature or a board location */
    private Object target;

    /** The direction the edu.cs2335.lab7.creature is turning to face */
    private int dir;

    /** One of the TURN_xxx constants indicating a left or right turn */
    private int rotateDir;




    /**
     * Creates a new CreatureMoveEvent
     *
     * @param eventType The event type
     * @param creature The edu.cs2335.lab7.creature making this move
     * @param target The target (board location to move to,
     *                           edu.cs2335.lab7.creature to infect, etc)
     */
    public CreatureMoveEvent(int eventType, DWCreature creature,
                             Object target) {

        this.eventType = eventType;
        this.creature = creature;
        this.target = target;
    }

    /**
     * Creates a new CreatureMoveEvent
     *
     * @param eventType The event type
     * @param creature The edu.cs2335.lab7.creature turning
     * @param dir The direction the edu.cs2335.lab7.creature is turning to face
     * @param rotateDir The direction the creature is rotating/turning
     *                  (either TURN_LEFT or TURN_RIGHT)
     */
    public CreatureMoveEvent(int eventType, DWCreature creature,
                             int dir, int rotateDir) {

        this.eventType = eventType;
        this.creature = creature;
        this.dir = dir;
        this.rotateDir = rotateDir;
    }




    /**
     * Gets the event type
     *
     * @return The event type
     */
    public int getEventType() {
        return eventType;
    }

    /**
     * Gets the target
     *
     * @return The target
     */
    public Object getTarget() {
        return target;
    }
    
    /**
     * Gets the direction the edu.cs2335.lab7.creature turned
     *
     * @return Direction turned
     */
    public int getDir() {
        return dir;
    }


    /**
     * Gets the direction the creature has to rotate
     *
     * @return The direction (TURN_LEFT or TURN_RIGHT)
     */
    public int getRotateDir() {
        return rotateDir;
    }


    /**
     * Gets the edu.cs2335.lab7.creature doing this move
     *
     * @return The edu.cs2335.lab7.creature
     */
    public DWCreature getCreature() {
        return creature;
    }

}

