
package edu.cs2335.lab7.creature;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * @author thaile
 * 
 * This is the brains of the DWCreature and tells it what to do
 */
public class Interpreter implements Serializable {
    /** The array of all the creature code */
    private ArrayList cCode;
    /** The creature the code is for */
    private DWCreature creature;
    /** The line that we are currently processing: 0 not used */
    private int lineNum;
    
    /**
     * Creates a new Interpreter using the given code
     * 
     * @param code The species code
     * @param c the creature this is linked to
     */
    public Interpreter(ArrayList code, DWCreature c) {
        cCode = code;
        lineNum = 1;
        
        creature = c;
    }
    
    /**
     * This method reads the code and tries to find the next command to execute
     * 
     * @return The command to execute
     */
    public String getNextCommand() {
        boolean found = false;
        String cmd;
        do {
            try { // error in case reached end of code go back to begining
                cmd = (String) cCode.get(lineNum);
            } catch (IndexOutOfBoundsException e) {
                lineNum = 1;
                cmd = (String) cCode.get(lineNum);
            } 
            StringTokenizer parser = new StringTokenizer(cmd);
            String token = parser.nextToken();
            if (token.equals("go")) {
                try {
                    int line = Integer.parseInt(parser.nextToken());
                    lineNum = line;
                } catch (NumberFormatException e) {
                    return "err 1";
                }
            } else if (token.equals("ifenemy")) {
                if (creature.getLocationInFront().isEnemy(creature)) {
                    try {
                        int line = Integer.parseInt(parser.nextToken());
                        lineNum = line;
                    } catch (NumberFormatException e) {
                        return "err 2";
                    }
                } else {
                    lineNum++;
                }
            } else if (token.equals("ifempty")) {
                if (creature.getLocationInFront().isEmpty()) {
                    
                    try {
                        int line = Integer.parseInt(parser.nextToken());
                        lineNum = line;
                    } catch (NumberFormatException e) {
                        return "err 3";
                    }
                } else {
                    lineNum++;
                }
            } else if (token.equals("ifsame")) {
                if (creature.getLocationInFront().isSame(creature)) {
                    
                    try {
                        int line = Integer.parseInt(parser.nextToken());
                        lineNum = line;
                    } catch (NumberFormatException e) {
                        return "err 4";
                    }
                } else {
                    lineNum++;
                }
            } else if (token.equals("ifwall")) {
                if (creature.getLocationInFront().isWall()) {
                    
                    try {
                        int line = Integer.parseInt(parser.nextToken());
                        lineNum = line;
                    } catch (NumberFormatException e) {
                        return "err 5";
                    }
                } else {
                    lineNum++;
                }
            } else { // either hop, infect, left , right
                found = true;
                lineNum++;
                cmd = token;
            }
        } while (!found);
        
        return cmd;
    }
    
    /**
     * This is for when the creature is infected
     * 
     * @param code
     *            The cCode to set.
     */
    public void setCCode(ArrayList code) {
        cCode = code;
        lineNum = 1;
    }
    
    /**
     * @return Returns the cCode.
     */
    public ArrayList getCCode() {
        return cCode;
    }
    
    /**
     * For testing our code
     * 
     * @param args
     *            command-line args.
     */
    public static void main(String[] args) {
    }
}