package edu.cs2335.lab7.data;

import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import edu.cs2335.lab7.dwboard.DWBoard;
import edu.cs2335.lab7.dwboard.DWBoardLocation;


/**
 * @author thaile
 *
 * creates a mapData object to send to the server
 */
public class MapMaker implements Serializable {
    /** Location of the map data */
    private String location;
    /** The object to be sent to the server */
    private MapData mapInfo;
   
    
    /**
     * Starts the map maker and creates a mapData object given the 
     * location of the map data;
     * @param location Where the map data is located
     */
    public MapMaker(String location) {
        this.location = location;
    }
    /**
     * reads in the *.cmp file along with the tiles
     * @return Int value corresponding to error or 1 on success
     */
    public int readInMap() {
        int[][] map;
        ImageIcon[] tiles;
        Image dirTile;
        File mapFile = new File(location + "/map.smp");
        BufferedReader input = null;
        int[][] mapMask;
        int mapSize;
        DWBoard board;
        try {  //Opens the map.smp to read
            input = new BufferedReader(new FileReader(mapFile));
        } catch (FileNotFoundException e) {
            return -1;
        }
        
        try {
            DWBoardLocation newLoc;
            mapSize = Integer.parseInt(input.readLine());
            mapMask = new int[mapSize][mapSize];
            board = new DWBoard(mapSize, mapSize);
            for (int i = 0; i < mapSize; i++) {
                final String mapLine = input.readLine();
                for (int j = 0; j < mapSize; j++) {
                    mapMask[i][j] = mapLine.charAt(j) - '0';
                    if ((mapLine.charAt(j) - '0') == 0) {

                        newLoc = new DWBoardLocation(j, i);
                        newLoc.setWall(true);
                        board.set(j, i, newLoc);
                    }
                }
            }
            input.close();
            mapFile.delete(); //Done with the map.smp
        } catch (IOException e) {
            return -2;
        } catch (NumberFormatException e) {
            return -2;
        }
        
        mapFile = new File(location + "/map.cmp");
        input = null;

        try {  //Opens the map.cmp to read
            input = new BufferedReader(new FileReader(mapFile));
        } catch (FileNotFoundException e) {
            return -3;
        }

        try {   //Reads in all the tiles
            int mapSize2 = Integer.parseInt(input.readLine());
            if (mapSize != mapSize2) { //invalid sizes maps
                return -6;
            }
            final int numtiles = Integer.parseInt(input.readLine());
            tiles = new ImageIcon[numtiles];
            BufferedImage sourceImage;
            String tileName = input.readLine();
            File dirFile = new File(location + "/" + tileName);
            
            GraphicsConfiguration graphicC = GraphicsEnvironment
            .getLocalGraphicsEnvironment().getDefaultScreenDevice()
            .getDefaultConfiguration();

            sourceImage = ImageIO.read(dirFile);
            dirTile = graphicC.createCompatibleImage(sourceImage.
                    getWidth(), sourceImage.getHeight(), 
                    Transparency.BITMASK);

            // reads in and stores the tiles in an array
            dirTile.getGraphics().drawImage(sourceImage, 0, 0, null);
            // deletes the file after ward
            dirFile.delete();
            Image tile;
            for (int i = 0; i < numtiles; i++) {
                tileName = input.readLine();
                File tileFile = new File(location + "/" + tileName);

                sourceImage = ImageIO.read(tileFile);
                tile = graphicC.createCompatibleImage(sourceImage.
                        getWidth(), sourceImage.getHeight(), 
                        Transparency.BITMASK);

                // reads in a stores the tiles in an array
                tile.getGraphics().drawImage(sourceImage, 0, 0, null);
                tiles[i] = new ImageIcon(tile);
                // deletes the file after ward
                tileFile.delete();
            }
            map = new int[mapSize][mapSize];

            for (int i = 0; i < mapSize; i++) { //note max of 10 tiles 
                final String mapLine = input.readLine();
                for (int j = 0; j < mapSize; j++) {
                    map[j][i] = mapLine.charAt(j) - '0';
                }
            }
            input.close();
            mapFile.delete(); //Done with the map.cmp
            
            File dir = new File(location);
            if (!dir.delete()) { //something still in folder
                File[] list = dir.listFiles();
                for (int i = 0; i < list.length; i++) {
                    System.out.println("deleting " + list[i].getName());
                    list[i].delete(); // so delete them all :-D
                }
                
                dir.delete(); // then delete the directory
            }
            input.close();
        } catch (IOException e) {
            return -4;
        } catch (NumberFormatException e) {
            return -5;
        }

        MapImage mapImage = new MapImage(map, tiles, new ImageIcon(dirTile));
        mapInfo = new MapData(location, mapSize, board, mapImage);
        return 1;
    }
    
    
    /**
     * @return Returns the mapInfo.
     */
    public MapData getMapInfo() {
        return mapInfo;
    }
}
