
package edu.cs2335.lab7.data;

import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.cs2335.lab7.creature.DWSpecies;

/**
 * @author thaile
 *
 * Creates a DWSpecies from a species files
 */
public class SpeciesMaker implements Serializable {
    /** Location of the map data */
    private String name;
    /** The object to be sent to the server */
    private DWSpecies species;
    
    /**
     * For testing the species maker class
     * @param args command-line args
     */
    public static void main(String[] args) {
        SpeciesMaker one = new SpeciesMaker("test");
        int err = one.readInSpecies();
        if (err != 1) {
            System.out.println("error code " + err);
        }
        
        DWSpecies species = one.getSpecies();
        
        JFrame test = new JFrame("test species");
        test.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        JPanel pane = new JPanel();
        
        pane.add(new JLabel(species.getIcon()));
        pane.add(new JLabel("test"));
        
        test.getContentPane().add(pane);
        test.pack();
        test.show();
        
    }
    
    /**
     * Starts the map maker and creates a mapData object given the 
     * location of the map data;
     * @param name Where the map data is located
     */
    public SpeciesMaker(String name) {
        this.name = name;
    }
    /**
     * reads in the *.cmp file along with the tiles
     * @return Int value corresponding to error or 1 on success
     */
    public int readInSpecies() {
        ArrayList script = new ArrayList();
        Image icon;
        
        File scriptFile = new File(name + "/script.dwl");
        BufferedReader input = null;

        try {  //Opens the map.cmp to read
            input = new BufferedReader(new FileReader(scriptFile));
        } catch (FileNotFoundException e) {
            return -1;
        }
        
        try {
            //This works because the script file starts at line 1 
            //not on line 0 as the ArrayList has it so we need 
            //to shift everything down 1
            String line = "script";
            do {
                script.add(line);
                line = input.readLine();
            } while (line != null);
                
            input.close();
            scriptFile.delete(); //Done with the script file
        } catch (IOException e) {
            return -2;
        }
        
        
        File imageFile = new File(name + "/img/");
        File[] images = imageFile.listFiles();
        int i;
        for (i = 0; i < images.length; i++) {
            if (images[i].getName().startsWith("icon")) {
                break;
            }
        }
        
       
            if (i >= images.length) { //didn't find any Icon images 
                return -3;
            } else {
                try {                
                    BufferedImage sourceImage;
                    File image = images[i];
                    GraphicsConfiguration graphicC = GraphicsEnvironment
                        .getLocalGraphicsEnvironment().getDefaultScreenDevice()
                        .getDefaultConfiguration();

                    sourceImage = ImageIO.read(image);
                    icon = graphicC.createCompatibleImage(sourceImage.
                        getWidth(), sourceImage.getHeight(), 
                        Transparency.BITMASK);

                    // reads in a stores the tiles in an array
                    icon.getGraphics().drawImage(sourceImage, 0, 0, null);
                    
                    //deletes the file afterwards
                    images[i].delete();
                    imageFile.delete();
                    
                    File mainFile = new File(name);
                    if (!mainFile.delete()) { //something still in folder
                        File[] list = mainFile.listFiles();
                        for (int j = 0; j < list.length; j++) {
                            list[i].delete(); // so delete them all :-D
                        }
                        mainFile.delete(); // then delete the directory
                    }

                } catch (IOException e) {
                    return -4;
                }
            }
        species = new DWSpecies(name, new ImageIcon(icon), script);
       
        return 1;
        
    }
    
    /**
     * @return Returns the species made.
     */
    public DWSpecies getSpecies() {
        return species;
    }
}
