package edu.cs2335.lab7.dwboard;

import java.io.Serializable;

/**
 * The Darwin's World game board.
 *
 * @author Brandyn Roemer Date: Mar 5, 2005 Time: 5:17:22 PM
 */
public class DWBoard implements Serializable {


    /** The number of columns in this board */
    private int ncols;

    /** The number of rows in this board */
    private int nrows;

    /** All the DWBoardLocations making up this board */
    private DWBoardLocation[][] gridLocations;

    /** The name of this board */
    private String name;

    /** The creator of this board */
    private String creator;






    /**
     * Creates a new DWBoard of the specified dimensions
     *
     * @param nrows The number of rows
     * @param ncols The number of columns
     */
    public DWBoard(int nrows, int ncols) {

        this.nrows = nrows;
        this.ncols = ncols;

        gridLocations = new DWBoardLocation[nrows][ncols];
        for (int i = 0; i < nrows; i++) {
            for (int j = 0; j < ncols; j++) {

                gridLocations[i][j] = new DWBoardLocation(i, j);
            }
        }

    }



    /**
     * Creates a new 0x0 DWBoard
     */
    public DWBoard() {

        this.nrows = 0;
        this.ncols = 0;

        gridLocations = new DWBoardLocation[0][0];

    }






    /**
     * Gets the DWBoardLocation on this board at the specified location
     *
     * @param row The row
     * @param col The column
     *
     * @return The DWBoardLocation at the specified location
     */
    public DWBoardLocation get(int row, int col) {

        if (row >= nrows || col >= ncols || row < 0 || col < 0
            || gridLocations == null) {

            return null;
        } else {

            return gridLocations[row][col];
        }

    }


    /**
     * Replaces the current DWBoardLocation at the specified row and column
     * with the provided DWBoardLocation.
     *
     * @param row The row
     * @param col The column
     * @param loc The new DWBoardLocation to insert
     */
    public void set(int row, int col, DWBoardLocation loc) {


        if (row >= nrows || row < 0 || col >= ncols || col < 0
            || gridLocations == null) {

            return;
        } else {

            gridLocations[row][col] = loc;
        }

    }


    /**
     * Gets the number of columns making up this board
     *
     * @return The number of columns on this board
     */
    public int getColCount() {
        return ncols;
    }

    /**
     * Gets the number of rows on this board
     *
     * @return The number of rows on this board
     */
    public int getRowCount() {
        return nrows;
    }

    /**
     * Gets all the DWBoardLocations making up this board
     *
     * @return All the DWBoardLocations making up this board
     */
    public DWBoardLocation[][] getGridLocations() {
        return gridLocations;
    }


    /**
     * Sets all the DWBoardLocations making up this board
     *
     * @param gridLocations All the DWBoardLocations making up this board
     */
    public void setGridLocations(DWBoardLocation[][] gridLocations) {
        this.gridLocations = gridLocations;
    }


    /**
     * Gets the name of this board
     *
     * @return The name of this board
     */
    public String getName() {
        return name;
    }


    /**
     * Sets the name of this board
     *
     * @param name The new name of this board.
     */
    public void setName(String name) {
        this.name = name;
    }


    /**
     * Gets the name of the creator of this board
     *
     * @return The name of the creator of this board
     */
    public String getCreator() {
        return creator;
    }


    /**
     * Sets the name of the creator of this board
     *
     * @param creator The name of the creator
     */
    public void setCreator(String creator) {
        this.creator = creator;
    }



    /**
     * Gets a printable String representation of this board
     *
     * @return A printable String representation of this board
     */
    public String toString() {

        String str;
        DWBoardLocation loc;



        str = "Name: " + (name != null ? name : "unknown") + "\n";
        str += "Created By: " + (creator != null ? creator : "unknown") + "\n";
        str += "Dimensions: " + nrows + " x " + ncols + "\n\n";


        if (gridLocations == null) {

            str += "[grid data not available]";
        } else {

            for (int i = 0; i < nrows; i++) {
                for (int j = 0; j < ncols; j++) {

                    loc = gridLocations[i][j];
                    if (loc == null) {

                        str += "?";
                    } else {

                        str += (loc.isWall() ? "1" : "0");
                    }

                }

                str += "\n";
            }

        } // ~else

        return str;
    }


}
