package edu.cs2335.lab7.dwboard;

import java.io.Serializable;

import edu.cs2335.lab7.creature.DWCreature;

/**
 * Represents a single location/square on a DWBoard  Each square
 * currently can be either a edu.cs2335.lab7.creature, a wall, or open
 *
 * @author Brandyn Roemer Date: Mar 5, 2005 Time: 5:27:53 PM
 */
public class DWBoardLocation implements Serializable {

    /** The edu.cs2335.lab7.creature on this spot (null if none) */
    private DWCreature creature;

    /** Tells whether this location represents a wall or not */
    private boolean wall;

    /** The row this location is in */
    private int row = -1;

    /** The column this location is in */
    private int col = -1;





    /**
     * Creates a new DWBoardLocation for use at the specified row/column
     * @param row The row location
     * @param col The column location
     */
    public DWBoardLocation(int row, int col) {

        creature = null;
        wall = false;

        this.row = row;
        this.col = col;
    }


    /**
     * Gets the edu.cs2335.lab7.creature at this location
     *
     * @return The creature at this location or null if there is none
     */
    public DWCreature getCreature() {
        return creature;
    }


    /**
     * Sets the edu.cs2335.lab7.creature at this location
     *
     * @param creature The creature at this location or null if there is none
     */
    public void setCreature(DWCreature creature) {

        this.creature = creature;
        wall = false;
    }


    /**
     * Tells whether this location is a wall or not
     *
     * @return True if a wall, false otherwise
     */
    public boolean isWall() {
        return wall;
    }


    /**
     * Tells whether this location is nothing or not
     *
     * @return True if empty, false otherwise
     */
    public boolean isEmpty() {
        return (creature == null && !wall);
    }

    /**
     * Tells whether this location holds an enemy or not.
     *
     * @param theCreature The creature checking if it's an enemy.
     * @return True if an enemy, false otherwise
     */
    public boolean isEnemy(DWCreature theCreature) {
        if (creature == null) {
            return false;
        } else {
            return !(creature.getSpecies().getSpeciesType().equals(
                   theCreature.getSpecies().getSpeciesType()));
        }
    }


    /**
     * Tells whether this location has a member of same species.
     *
     * @param theCreature The creature checking if it's the same.
     * @return True if same species, false otherwise
     */
    public boolean isSame(DWCreature theCreature) {
        if (creature == null) {
            return false;
        } else {
            return (creature.getSpecies().getSpeciesType().equals(
                   theCreature.getSpecies().getSpeciesType()));
        }
    }

    /**
     * Set whether this location is a wall or not
     *
     * @param wall True if wall, false otherwise
     */
    public void setWall(boolean wall) {
        this.wall = wall;
    }



    /**
     * Determines if this spot is open or not
     *
     * @return True if this spot is open, false otherwise
     */
    public boolean isOpen() {

        return (!wall && creature == null);
    }




    /**
     * Gets the row
     *
     * @return The row
     */
    public int getRow() {
        return row;
    }


    /**
     * Sets the row
     *
     * @param row The row
     */
    public void setRow(int row) {
        this.row = row;
    }


    /**
     * Gets the column
     *
     * @return The column
     */
    public int getCol() {
        return col;
    }


    /**
     * Sets the column
     *
     * @param col The column
     */
    public void setCol(int col) {
        this.col = col;
    }
}
