package edu.cs2335.lab7.server;

import java.util.ArrayList;
import java.util.List;

import edu.cs2335.lab7.creature.CreatureMoveEvent;
import edu.cs2335.lab7.creature.CreatureMoveListener;
import edu.cs2335.lab7.creature.DWCreature;
import edu.cs2335.lab7.creature.DWSpecies;
import edu.cs2335.lab7.data.CreatureData;
import edu.cs2335.lab7.data.MapData;
import edu.cs2335.lab7.dwboard.DWBoard;
import edu.cs2335.lab7.dwboard.DWBoardLocation;

/**
 * Darwin's World - CS2335 Lab7
 * @version 1.0
 * 
 * Game - Game model class. Holds players in a game and other associated data.
 */
public class Game implements Runnable, CreatureMoveListener {
    /**Name used to distinguish this game.*/
    private String name;
    /**A list of users (server sockets) connected to this game.*/
    private List participants;
    /**List of creatures in this game */
    private DWCreature[] creatures;
    /**Species 1 of the game */
    private DWSpecies species1 = null;
    /**Species 2 of the game */
    private DWSpecies species2 = null;
    /**Number of users ready for the next turn*/
    private int readyUsers;
    /** Holds all the infomation about the map */
    private MapData mapInfo = null;
    /** DATA: temp holds all the moves for a current turn */
    private ArrayList moves;
    /** DATA: holds the number of creatures per side */
    private int numCreatures = 0;
    /** DATA: holds the board */
    private DWBoard board;
    /** DATA: number of creatures of species 1 */
    private int numOfSpecies1;    
    /** DATA: number of turns in game */
    private int numOfTurns;
    /** DATA: the winning species */
    private DWSpecies winner = null;
    /** DATA: game data buffer */
    private GameData dataBuffer;
    /** FLAG: initilizes the game stuff */
    private boolean initGame = false;
    /** FLAG: for done processing a turn */
    private boolean turnCalcDone = false;
    /** FLAG: noWinner */
    private boolean noWinner = true;
    
    /** FLAG: and playing */
    private boolean playing = true;
    /** The server game */
    private boolean servergame;
    
    /**
     * This runs the game itself
     */
    public void run() {
        
        //must have called initGame
        while (initGame && noWinner && playing) {
            
            try {
                Thread.sleep(250);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            
            // Calculates the moves for the creatures
            if (!turnCalcDone) {
                //gives everycreature a turn
                numOfTurns++;
                for (int i = 0; i < 2 * numCreatures; i++) {
                    creatures[i].doTurn();
                }
                turnCalcDone = true;
                updateBuffer();
            }
            
        }
        
        
        // update the wins/losses
        // added by varkalas, make sure this doesn't break anything :P
        // species1, species2
        if (servergame) {
            UserDatabase db = ((ServerConnection) 
                    participants.get(0)).getDatabase();
            User user1 = (User) db.getUser(((ServerConnection)
                    participants.get(0)).getUsername());
            User user2 = (User) db.getUser(((ServerConnection) 
                    participants.get(1)).getUsername());
            for (int n = 0; n < user1.getStable().size(); n++) {
                DWSpecies temp = (DWSpecies) user1.getStable().get(n);
                if (species1.getSpeciesType().equals(temp.getSpeciesType())) {
                    if (winner.getSpeciesType().equals(temp.getSpeciesType())) {
                        // wins
                        user1.won();
                        temp.won();
                    } else {
                        // loses
                        user1.lost();
                        temp.lost();
                    }
                } else if (species2.getSpeciesType().equals(
                        temp.getSpeciesType())) {
                    if (winner.getSpeciesType().equals(temp.getSpeciesType())) {
                        // wins
                        user1.won();
                        temp.won();
                    } else {
                        // loses
                        user1.lost();
                        temp.lost();
                    }
                }
            }
            for (int n = 0; n < user2.getStable().size(); n++) {
                DWSpecies temp = (DWSpecies) user2.getStable().get(n);
                if (species1.getSpeciesType().equals(temp.getSpeciesType())) {
                    if (winner.getSpeciesType().equals(temp.getSpeciesType())) {
                        // wins
                        user2.won();
                        temp.won();
                    } else {
                        // loses
                        user2.lost();
                        temp.lost();
                    }
                } else if (species2.getSpeciesType().equals(
                        temp.getSpeciesType())) {
                    if (winner.getSpeciesType().equals(temp.getSpeciesType())) {
                        // wins
                        user2.won();
                        temp.won();
                    } else {
                        // loses
                        user2.lost();
                        temp.lost();
                    }
                }
            }
            
        }
        
        synchronized (moves) {
            moves.add(0 + ",winner," + winner.getSpeciesType());
        }
        while (!moves.isEmpty()) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        if (participants.get(0) != null) {
            ((ServerConnection) participants.get(0)).endGame();
        }
    }
    
    
    /**
     * Updates the data buffer each turn
     */
    private void updateBuffer() {
        CreatureData[] theData = new CreatureData[2 * numCreatures];
        for (int k = 0; k < 2 * numCreatures; k++) {
            theData[k] = creatures[k].getData();
        }
        //Create a new data object
        dataBuffer = new GameData(mapInfo.getMapImage(), species1,
                               species2, numCreatures, theData);
    }
    
    /**
     * Wheather we're still playing or not
     * @param value true or false
     */
    public void setPlaying(boolean value) {
        playing = value;
    }
    
    /**
     * This creates a GameData object to send to the client
     * @return A GameData object
     */
    public synchronized GameData getGameData() {
        if (!initGame) { //if any info missing don't send
            return null;
        } 
        
        return dataBuffer;
    }
    
    
    
    
    /**
     * This sets up all the creature and board into there
     * inital states
     * @return An error code or 1 on success
     */
    public int initGame() {
        if (species1 == null) {
            return -1;
        } else if (species2 == null) {
            return -2;
        } else if (mapInfo == null) {
            return -3;
        } else if (numCreatures == 0) {
            return -4;
        } else {
            
            //STEP 1: create all the creatures for the battle
            int i;
            creatures = new DWCreature[2 * numCreatures];
            for (i = 0; i < numCreatures; i++) {
                creatures[i] = new DWCreature(i, species1);
            }
            for (i = numCreatures; i < 2 * numCreatures; i++) {
                creatures[i] = new DWCreature(i, species2);
            }
            
            //STEP 2: Create a new board 
            board = mapInfo.getBoard();
            
            //STEP 3: Randomly place the creatures on the board
            for (int j = 0; j < 2 * numCreatures; j++) {
                boolean placed = false;
                int x;
                int y;
                int dir;
                DWBoardLocation loc;
                CreatureData cData;
                int value = mapInfo.getSize() - 1;
                while (!placed) {
                    x = Math.round((float) Math.random() * value);
                    y = Math.round((float) Math.random() * value);
                    loc = board.get(x, y);
                    if (loc.isOpen()) {
                        dir = Math.round((float) Math.random() * 3);
                        dir = dir * 90;
                        DWSpecies specie = creatures[j].getSpecies();
                        //create a new creature data object
                        cData = new CreatureData(specie.getSpeciesType(), 
                                x, y, dir);
                        creatures[j].setData(cData);
                        creatures[j].setBoard(board);
                        creatures[j].addCreatureMoveListener(this);
                        loc.setCreature(creatures[j]);
                        placed = true;
                    }
                }
                
            }
            // STEP5: init the dataBuffer
            // get the latest creature data from the DWCreatures
            CreatureData[] theData = new CreatureData[2 * numCreatures];
            for (int k = 0; k < 2 * numCreatures; k++) {
                theData[k] = creatures[k].getData();
            }
            //Create a new data object
            dataBuffer = new GameData(mapInfo.getMapImage(), species1, species2,
                    numCreatures, theData);
            
            
            //STEP 4: init the move arrays
            moves = new ArrayList();
            numOfTurns = 0;
            numOfSpecies1 = numCreatures;
            initGame = true;
            noWinner = true;
            Thread game = new Thread(this);
            game.start();
            return 1;
        }
    }
    
    /**
     * Default constructor, creates game object.
     * @param name The name of this game
     */
    public Game(String name) {
        this.name = name;
        participants = new ArrayList();
        servergame = false;
    }
    
    
    
    /**
     * @param numCreatures The numCreatures to set.
     */
    public synchronized void setNumCreatures(int numCreatures) {
        this.numCreatures = numCreatures;
    }
    /**
     * Converts from a arraylist to string array
     * only if all turns are processed 
     * @return Returns the moves.
     */
    public synchronized String getCmds() {
        if (turnCalcDone) {
            System.out.println("size: " + moves.size());
            String cmd = "GAMEMOVES~";
            for (int i = 0; i < moves.size(); i++) {
                cmd = cmd + (String) moves.get(i) + "~";
            }
            cmd = cmd + "ME";
            
            // get the latest creature data from the DWCreatures
            CreatureData[] cData = new CreatureData[2 * numCreatures];
            for (int i = 0; i < 2 * numCreatures; i++) {
                cData[i] = creatures[i].getData();
            }
            //Create a new data object
            dataBuffer = new GameData(mapInfo.getMapImage(), species1, species2,
                    numCreatures, cData);
            
            return cmd;
        }
        return null;
        
    }
    
    /**
     * Just get the first command from the queue
     * @return A string or null
     */
    public synchronized String getACmd() {
        String cmd = null;
        try {
            cmd = (String) moves.remove(0);
            if (moves.isEmpty()) {
                turnCalcDone = false;
            }
        } catch (IndexOutOfBoundsException e) {
            turnCalcDone = false;
        }
        
        return cmd;
    }
    /**
     * @param species1 The species1 to set.
     */
    public synchronized void setSpecies1(DWSpecies species1) {
        this.species1 = species1;
    }
    /**
     * @param species2 The species2 to set.
     */
    public synchronized void setSpecies2(DWSpecies species2) {
        this.species2 = species2;
    }
    /**
     * @param mapInfo The mapInfo to set.
     */
    public synchronized void setMapInfo(MapData mapInfo) {
        this.mapInfo = mapInfo;
    }
    /**
     * Adds a new person to the game.
     * @param usertoadd  connection of the joining player
     */
    public synchronized void addToParticipants(ServerConnection usertoadd) {
        participants.add(usertoadd);
    }
    
    /**
     * Removes a user form the current game.
     * @param usertoremove the  exiting user
     */
    public synchronized void removeFromParticipants(ServerConnection 
            usertoremove) {
        ServerConnection curruser = null;
        for (int i = 0; i < participants.size(); i++) {
            curruser = (ServerConnection) (participants.get(i));
            if ((curruser.getUsername()).equals(usertoremove.getUsername())) {
                participants.remove(i);
            }
        }
    }
    
    /**
     * handles the creature move events
     * @param e A creature move event
     */
    public synchronized void handleCreatureMove(CreatureMoveEvent e) {
        int type = e.getEventType();
        if (type == 1) { //hop
            moves.add(e.getCreature().getId() + ",hop");
        } else if (type == 2) { //infect
            DWCreature temp = (DWCreature) e.getTarget();
            moves.add(e.getCreature().getId() + ",infect," + temp.getId());
            System.out.println("INFECT " + e.getCreature().getId() + ": " 
                    + numOfSpecies1);
            //basically if the infecting species is of species 1
            //then increment counter
            if (e.getCreature().getSpecies().getSpeciesType().
                    equals(species1.getSpeciesType())) {
                numOfSpecies1++;
                System.out.println("Number of species1: " + numOfSpecies1  
                        + ": " + numCreatures);
                if (numOfSpecies1 >= 2 * numCreatures) {
                    System.out.println("winner");
                    noWinner = false;
                    winner = species1;
                }
            } else {
                numOfSpecies1--;
                System.out.println("Number of species1: " + numOfSpecies1
                        + ": " + numCreatures);
                if (numOfSpecies1 <= 0) {
                    System.out.println("winner");
                    noWinner = false;
                    winner = species2;
                }
            }
        } else if (type == 3) { //turn
            moves.add(e.getCreature().getId() + ",turn," + e.getDir());
        } // type == 4  do nothing infected by?
    }
    
    /**
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }
    
    /**
     * @return Returns the participants.
     */
    public List getParticipants() {
        return participants;
    }
    
    /**
     * @param name The name to set.
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * @return Returns the ready users.
     */
    public int getReadyUsers() {
        return readyUsers;
    }
    
    /**
     * Adds one to the readyUser counter.
     */
    public synchronized void addReadyUser() {
        readyUsers++;
        if (readyUsers == participants.size() && turnCalcDone) {
            for (int i = 0; i < readyUsers; i++) {
                String cmd = getCmds();
                if (cmd != null) {
                    ((ServerConnection) participants.get(i)).sendMessage(cmd);
                }
                
            }
            
            while (!moves.isEmpty()) {
                moves.remove(0);
            }
            turnCalcDone = false;
            readyUsers = 0;
            
        }
    }
    
    /**
     * Resets the readyUser counter.
     */
    public synchronized void resetReadyUsers() {
        readyUsers = 0;
    }
    
    /**
     * @return Returns the species1.
     */
    public DWSpecies getSpecies1() {
        return species1;
    }
    
    /**
     * @return Returns the species2.
     */
    public DWSpecies getSpecies2() {
        return species2;
    }
    
    /**
     * @param serverGame The servergame to set.
     */
    public synchronized void setServerGame(boolean serverGame) {
        this.servergame = serverGame;
    }
}
