package edu.cs2335.lab7.server;

import java.io.Serializable;

import edu.cs2335.lab7.creature.DWCreature;
import edu.cs2335.lab7.creature.DWSpecies;
import edu.cs2335.lab7.data.CreatureData;
import edu.cs2335.lab7.data.MapImage;

/**
 * @author macguy
 *
 * Object send to client containing all the current game data
 * Created everytime server needs to send the info
 */
public class GameData implements Serializable {
    /** Contains all info for drawing the map */
    private final MapImage map;
    /** Info on the first species */
    private final DWSpecies species1;
    /** Info on the second species */
    private final DWSpecies species2;
    /** Info on the number Of creatures */
    private final int numcreatures;
    /** Info on the creatures data */
    private final CreatureData[] data;
    /** The creature */
    private DWCreature[] creatures;
    
    /**
     * Setup the game data to be send to the client
     * @param map The map infomation
     * @param species1 the first species
     * @param species2 the second species
     * @param numcreatures the number of creatures on the map
     * @param data The creatures data
     */
    public GameData(MapImage map, DWSpecies species1,
            DWSpecies species2, int numcreatures, CreatureData[] data) {
        this.map = map;
        this.species1 = species1;
        this.species2 = species2;
        this.numcreatures = numcreatures;
        this.data = data;
        
    }    
    
    /**
     * Setup the game data to be send to the client
     * @param map The map infomation
     * @param species1 the first species
     * @param species2 the second species
     * @param numcreatures the number of creatures on the map
     * @param data The creatures data
     * @param creatures the creature
     */
    public GameData(MapImage map, DWSpecies species1,
            DWSpecies species2, int numcreatures,
            CreatureData[] data, DWCreature[] creatures) {
        
        this.map = map;
        this.species1 = species1;
        this.species2 = species2;
        this.numcreatures = numcreatures;
        this.data = data;
        this.creatures = creatures;
    }
    
    /**
     * @return Returns the data.
     */
    public CreatureData[] getData() {
        return data;
    }
    /**
     * @return Returns the map.
     */
    public MapImage getMap() {
        return map;
    }
    /**
     * @return Returns the numcreatures.
     */
    public int getNumcreatures() {
        return numcreatures;
    }
    /**
     * @return Returns the species1.
     */
    public DWSpecies getSpecies1() {
        return species1;
    }
    /**
     * @return Returns the species2.
     */
    public DWSpecies getSpecies2() {
        return species2;
    }
}
