package edu.cs2335.lab7.server;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Darwin's World - CS2335 Lab7
 * @version 1.0
 * 
 * UserDatabase - Stores all the user objects and user information.
 */
public class UserDatabase implements Serializable {
    
    /**Success code.*/
    private static int success = 1;
    
    /**Code to indicate an invalid user was specified by some action. */
    private static int nouser = 2;
    
    /**Failure code that indicates another user with the same name was found.*/
    private static int duplicateuser = 5;
    
    /**Failure code that indicates the given password was invalid.*/
    private static int badpass = 6;

    /**A list of all user objects.*/
    private List userlist;
    
    /**
     * Constructor that initializes the userlist.
     */
    public UserDatabase() {
        userlist = new ArrayList();
    }
    
    /**
     * Registers a new user with the given username and password.
     * @param username  the user's name
     * @param password  the users password
     * @return duplicateuser  a user with that name already exists
     *         success  user registered successfully
     */
    public synchronized int registerNewUser(String username, String password) {
        String currusername;
        for (int i = 0; i < userlist.size(); i++) {
            currusername = ((User) (userlist.get(i))).getUsername();
            if (currusername.equals(username)) {
                return duplicateuser;
            }
        }
        userlist.add(new User(username, password));
        return success;
    }
    
    /**
     * Logs a user in to the system if they provided a valid username/password.
     * @param username  the login name
     * @param password  the password
     * @return success  login successful
     *         badpass  invalid password
     *         nouser  invalid username
     */
    public synchronized int loginUser(String username, String password) {
        User curruser;
        for (int i = 0; i < userlist.size(); i++) {
            curruser = (User) (userlist.get(i));
            if ((curruser.getUsername()).equals(username)) {
                if ((curruser.getPassword()).equals(password))  {
                    return success; 
                } else {
                    return badpass;
                }
            }
        } return nouser;
    }
    
    /**
     *Logs off all users by setting their status to offline.
     *Used when the server exits.
     */
    public synchronized void logoffAllUsers() {
        for (int i = 0; i < userlist.size(); i++) {
            ((User) (userlist.get(i))).setStatus("offline");
        }
    }
    
    /**
     * Changes the status of a user (active/offline).
     * @param username  the user to change
     * @param newstatus  the user's new status
     */
    public synchronized void changeStatus(String username, String newstatus) {
        User curruser;
        for (int i = 0; i < userlist.size(); i++) {
            curruser = (User) (userlist.get(i));
            if ((curruser.getUsername()).equals(username)) {
                curruser.setStatus(newstatus);
                return;
            }
        }
    }

    /**
     * Returns the user object with the username as given.
     * @param username  the username of the user to get
     * @return curruser  the user
     *         null  no user with that name found
     */
    public synchronized User getUser(String username) {
        User curruser = null;
        for (int i = 0; i < userlist.size(); i++) {
            curruser = (User) (userlist.get(i));
            if ((curruser.getUsername()).equals(username)) {
                return curruser;
            }
        }
        return null;
    }

    /**
     * @return Returns the list of users.
     */
    public synchronized List getUserList() {
        return userlist;
    }
}
